/***************************************************************************
* Demoprogramm M8SRQTST.C fr die Verwendung der Modular-4/486 PC-Bibliothek.
*
* Version: 1.E.        Erstellung: 28.08.1993
*                      letzte nderung: 06.10.1995
*                      Autor: BF, MIH
*
* Demonstriert wird insbesondere die Auslsung von Service-Requests durch
* die Modular-4/486 Karte auf dem PC. Es wird das Echtzeitprogramm
* M8P0307.EXE bentigt.
*
* Dieses Programm wurde mit Borland C++ 3.1 getestet.
***************************************************************************/

#include "ml8bib.h"
#include "conio.h"
#include "stdio.h"
#include "dos.h"
#include "stdlib.h"
#include "string.h"

#define TRUE	1
#define FALSE	0

#define MODE       1              /* mit PC-Interrupt */
#define BLINKRATE  0x5000         /* Blinkrate fr Echtzeitprogramm */
#define IRQCOUNT   10             /* Anzahl der Blinkphasen */

#define RTPROG  "\\sorcus\\ml8\\rt\\pascal\\beisp\\M8P0307.EXE"
#define OS      "ROM"

void far pascal errorhandler(void);
void far pascal interrupt_service(void);

ushort	    irq_count= 0;	/* IRQ-Zhler, global deklariert */
ushort	    irq_count_main= 0;  /* Bei Differenz zu irq_count:   */
				/* Anzeige in main aktualisieren */
ushort	    message= 0xFFFF;   	/* Aktuelle Nachricht von der ML8-Karte */
byte        mode=MODE;          /* mit/ohne PC-Interrupt */
ushort      sema_printf=0;      /* in Fehlerbehendlung sicherstellen, da
				   bei Aufruf von Benutzer-Service-Funktion aus
				   keine Bildschirmausgabe stattfindet */
ushort      srqerror = 0;       /* Fehler in SRQ-Routine aufgetreten. Dieser
				   mu in der Schleife der Main-Funktion
				   abgefangen werden */

int main(int argc,char *argv[])
{
   ushort	w;
   ushort	pgm,
		task,
		irq_channel;
   char		s[80];

#pragma pack(1)	/* Achtung! Programm M2P0307.exe verwendet Byte-Alignment */

   struct {
	byte	status;		/* Status des Programms */
	ushort	blink_rate;     /* Blinkrate */
	byte	led_status;	/* Zustand der LED */
	byte	led;           	/* interne/externe LED */
	ushort	countdown;	/* Anzahl On-Phasen, 0 = keine, 0xFFFF = unendlich*/
	byte	with_srq;	/* PC-Interrupt auslsen, 1=TRUE,0=FALSE*/
	ushort	srq_sign;	/* Request-Zeichen */
	byte	srq_pending;    /* Request wartet */
   } parameter;


  cprintf("Beispielprogramm M8SRQTST.C\n\r");


  ml8_bib_startup(LANGUAGE_C);  /* Verwendete Sprache: C */
				/* Bibliothek fr diese Karte initialisieren */
				/* und Reset der Karte */
  irq_channel = 7;
  w=ml8_reset(0, 0x380, mode, irq_channel, 1, * errorhandler, OS, 10);
  if (w>0)
  {
	printf("Fehler bei ml8_reset:\n\r");
	ml8_get_error_message(s);
	printf("%s; Rckgabewert: %04X\n\r",s, w);
	ml8_exit_card(0);
	return(1);
  }
  ml2_get_os_version(s);
  cprintf("Betriebssystem der Karte: %s\n\r",s);

  ml8_set_service(interrupt_service);

  pgm = 0x0307;
  task = 16;      /* Programm M8P0307.EXE zur Karte laden */
  w=ml8_transfer_and_install( RTPROG, 0, task, pgm, 0, 0x1980, 0);
  if (w>0)
  {
	ml8_get_error_message(s);
	printf("Fehlermeldung von ml8_transfer_and_install:\n\r");
	printf("Fehler %04Xh: %s\n\r",w ,s );
	ml8_exit_card(0);
	return(1);
  }

  ml8_suspend_requests();

  parameter.status = 1;
  parameter.blink_rate = BLINKRATE;
  parameter.led_status = 0;
  parameter.led = 1;
  parameter.countdown = IRQCOUNT;
  parameter.with_srq = TRUE;
  parameter.srq_sign = 0x0080;
  parameter.srq_pending = FALSE;

	    /* Parameter setzen, Programm wird dadurch auch gestartet;
	       Interrupts werden erst nach ml8_allow_request() empfangen */
  ml8_write_par_block(task, 0, sizeof(parameter), &parameter);

  ml8_allow_requests();   /*Requests von der Multi-LAB/2 Karte freigeben */
  cprintf("Fr jeden Interrupt ein Sternchen auf dem Bildschirm:\n\r");

  do
  {
     if ((irq_count != irq_count_main) && (!srqerror)) {
	/* wird die Blinkrate erhht, so werden nicht mehr alle Interrupts auf
	   dem Bildschirm protokolliert, da die Benutzer-Service-Funktion
	   pro Schleifendurchlauf mehrfach aufgerufen werden kann! */
	 irq_count_main= irq_count;
	 cprintf("\n\r* Message: %04Xh   Nummer: %d", message, irq_count);
     }
     if (srqerror){
	cprintf("\n\r\n\r *** Fehler in Interrupt-Service-Routine aufgetreten; Abbruch ***\r\n");
	ml8_suspend_requests();    /* weitere Interrupts unterbinden */
	ml8_exit_card(0);
	return(1);
     }
  } while ( (!kbhit()) && (irq_count_main<IRQCOUNT) );

  ml8_suspend_requests();
  cprintf("\n\r");
  if (irq_count==IRQCOUNT)
	cprintf("Alle %d Service Requests empfangen\n\r",IRQCOUNT);
  else
	cprintf("Es sind nur %d Service Requests empfangen worden\n\r", irq_count);

  ml8_exit_card(0);

  return(0);
}

/* Benutzer-Service-Funktion,
 * wird von der Bibliothek als Teil der Hardware-Interrupt-Service-Routine
 * aufgerufen.
 * !!!! Achtung: keine DOS-Funktionen aufrufen, da nicht reentrant!!!!
 * Da in der Fehler-Behandlungsfunktion "printf" benutzt wird, mu ber einen
 * Schalter ausgeschlossen werden, da bei Aufruf der Fehler-
 * Behandlungsfunktion aus der Benutzer-Service-Funktion heraus
 * "printf"-Befehle ausgefhrt werden.
 */
void far pascal interrupt_service(void)
{
   ushort i;

   sema_printf=1;
   message = ml8_get_message();
   ml8_read_par_block(16, 0, sizeof(i), &i);/* Dummy-Lesezugriff auf Task-
					       Parameter */
   irq_count++;
   sema_printf=0;
}


/* Fehlerbehandlungsroutine: wird als Callback-Funktion von der Bibliothek
   aus aufgerufen, falls ein Fehler aufgetreten ist */
void far pascal errorhandler(void)
{
   struct ml8_error_info_type  error_info, *err_p=&error_info;
   char	s80[81];
   char	ch;
   ushort	w;

    /* falls Fehlerbehandlung aus der SRQ-Routine aufgerufen wurde, darf
       keine Bildschirmausgabe stattfinden. In diesem Fall wird das
       Programm nach Abfrage der von "srqerror" in der Hauptroutine beendet */

   if ( (sema_printf) ){
      srqerror=1;
      return;
   }

   ml8_get_error_info(err_p);

   if (err_p->mark==0) return;

   cprintf("\n\r\n\rBenutzer-eigene Fehlerbehandlung\n\r\n\r");

   if (err_p->ok)
   {
      cprintf("Die Fehlerbehandlung wurde grundlos aufgerufen\n\r");
      cprintf("(mglicherweise direkt vom C-Programm, statt von der Bibliothek)\n\r");
      getch();
      return;
   }
   if (err_p->errorclass==5)
   {
      w=ml8_get_message();

      cprintf("Die Fehlermeldung wurde durch einen Errorrequest\n\r");
      cprintf("von der Multi-LAB/2 Karte ausgelst\n\r\n\r");
      cprintf("Fehler-Request von Karte: ");
      cprintf("     SRQ: %X>H\n\r",w);
      cprintf("Bitte eine Taste drcken ");

      while (!kbhit()) ;
	  return;
   }
   cprintf("\n\r");
   if (err_p->statusmap & 128 > 0)
   {
     cprintf("Kommunikationsversuch ohne Initialisierung der \n\r");
     cprintf("Bibliothek fr die angewhlte Karte\n\r");
     cprintf("Verwenden Sie ml8_start oder ml8_reset\n\r");
     cprintf("Programm angehalten\n\r");
     abort();
   }
   if (err_p->statusmap & 1 > 0)
   {
     cprintf("Kommunikationsversuch ohne ausreichende Fehlerbehandlung.\n\r");
     cprintf("Verwenden Sie ml8_clear_error in der Fehlerbehandlungsroutine\n\r");
     cprintf("Die Fehlervariablen enthalten noch den ursprnglichen Fehler.\n\r\n\r");
   }
   ml8_get_error_message(s80);
   cprintf("Fehlernachricht: %s\n\r",s80);
   cprintf("Fehlerklasse %d     ",err_p->errorclass);
   cprintf("Fehler-Code %d\n\r",err_p->errorcode);
   cprintf("Fehler aufgetreten bei Ausfhrung von Makro-Befehl %XH\n\r",err_p->outwords[0]);
   cprintf("Schnittstellenstatus (Basisadresse) %XH\n\r",err_p->statusport);

   cprintf("\n\rFehler aufgetreten nach Markierung %d\n\r",err_p->mark);
   ml8_get_mark_message(s80);
   cprintf("Mark-Nachricht: %s\n\r",s80);

   cprintf("\n\rProgramm weiterfhren <RETURN> oder Abbruch <A/a> ? ");

   ch='x';
   while (strchr("Aa'\r'", ch)==0) ch=getch();
   if (strchr("Aa", ch)>0)
   {
      cprintf("\n\rProgramm wird angehalten\n\r");
      ml8_exit_card(0);
      abort();
   }
   else
   {
      cprintf("\n\rProgramm wird weitergefhrt\n\r");
      ml8_clear_error();
      ml8_get_error_message(s80);
      if (strcmp(s80, "")!=0)
      {	printf("ml8_clear_error() lscht die error_message nicht\n\r");
      }
      w=ml8_clear();
      if (w>0)
      {
	 cprintf("\n\rml8_clear ist erfolglos. Programm wird angehalten\n\r");
	 ml8_exit_card(0);
	 abort();
      }
   }
}





