/****************************************************************************/
/**                       Beispielprogramm fr                             **/
/**                Analogdatenerfassung mit M-5B-1/U                       **/
/**                           Version 1.A.                                 **/
/**									   **/
/**			   Dateiname : MESS030C.C			   **/
/**                 Erstellungsdatum : 30.09.93                            **/
/**                  Letzte nderung : 24.04.96 (MIH)                      **/
/**                           Status : freigegeben                         **/
/**                            Autor : G. Rudolf                           **/
/**                                                                        **/
/**                                                                        **/
/**       (c) 1993..1996 SORCUS Computer GmbH  Heidelberg                  **/
/**                                                                        **/
/****************************************************************************/
/** Kurzbeschreibung: Das Programm installiert das Echtzeitprogramm 30CH,  **/
/** mit 4 Kanle mit einer Abtastrate von 10 kHz und stellt sie in einer  **/
/** einfachen Graphik dar (Beenden der Messung durch die Taste 'E')        **/
/****************************************************************************/
/*
*  Programmart: PC-Programm
*  Funktion: Beispielprogramm zur Analogdatenerfassung mit Verwendung des
*  	  Echtzeitprogrammes M8P030C.EXE
*  Programmiersprache: Borland C++ Version 3.1
*  Bibliotheksversion: ab ML8BIB 1.C  und hher
*  Betriebssystemversion: ab ML8-2D.09  und hher
*  Verwendetes Echtzeitprogramm: M8P030C.EXE
*  Zugehrige Installationsdatei: -
*  Hardware-Voraussetzungen: Modul M-5B-1/U auf Steckplatz 1
*  Software-Voraussetzungen: Das Programm M8P030C.EXE mu im Verzeichnis
*			     '\\sorcus\\ml8\\rt\\bc\\beisp' und der
*			     BGI-Treiber im aktuellen Verzeichnis stehen.
*
*  Anmerkung: Dieses Beispiel zeigt wie mit der C-Bibliothek ML8BIB ein
*  	   Echtzeit-Programm auf der MODULAR-4/486 installiert und
*  	   gestartet wird. Auerdem wird demonstriert, wie Daten aus dem
*  	   Datenbereich einer Task zum PC transferiert werden knnen (polling).
*  	   Die Grafikausgabe ist sehr einfach gehalten. Fr Programme mit
*  	   ansprechender Gestaltung empfehlen wir die Verwendung der
*  	   Grafiktoolbox GA1 (nur fr Pascal), mit der komfortable
*  	   Grafikausgabe in einfacher Weise realisiert werden kann.
*
*  Bedienung: Nach dem Starten wird ein Reset der MODULAR-4/486 durchgefhrt und
*  	   das Echtzeitprogramm installiert. Danach wird die erste Messung
*  	   durchgefhrt (4 Kanle mit je 10 kHz, 1000 Messwerte pro Kanal).
*  	   Wenn das Messergebnis auf dem Bildschirm dargestellt ist, wird
*  	   durch Drcken einer beliebigen Taste die nchste Messung gestar-
*  	   tet. Mit der Taste 'E' wird das Programm beendet.
*
*  Querverweis: MESS030C.C, M8P030C.C
***************************************************************************/
#include <dos.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <graphics.h>
#include "ml8bib.h"

#define MP_NAME      "\\sorcus\\ml8\\rt\\bc\\beisp\\M8P030C.EXE"      
					/*Name des Echtzeitprogrammes*/
#define MP_NR        0x30C              /*Nummer des Echtzeitprogrammes*/
#define MP_INT       0x91               /*Interrupt des Messprogramms (Timer-A)*/
#define MP_FLAG      0x1982             /*Installierungsflags des Echtz.pgms*/
#define MESS_TASK    16                 /*Als Task 16 installieren*/

#define ML8_ADR      0x380              /*MODULAR-4/486 Basisadresse*/
#define MODUL        1                  /*M-SH12-8 auf Steckplatz 1*/
#define ANZ_KAN      4                  /*4 Kanle einlesen*/
#define ANZ_MESS     1000               /*1000 Messungen pro Kanal*/
#define ABTAST_FREQ  10000              /*Abtastrate = 10 kHz*/
#define TIMER_TAKT   2.5E6              /*Timereingangstakt der ML8 = 2,5 MHz*/
#define DATA_SIZE    (long) ANZ_MESS*ANZ_KAN*2   /*Gre des zu reservierenden Datenbereichs*/


      /*--------------------- Statusdefinitionen ---------------------------*/
#define RUNNING    	2
#define FINISHED   	3
#define STOPPED    	4
#define ERROR      	0x80

#define ML8_OS   	"ROM"           /*Betriebssystem der MODULAR-4*/
#define BGI_PATH  	""              /*Suchpfad zu Grafikdateien*/
#define TEXT_MODE_ON  	1
#define GRAPH_MODE_ON 	2

/*----------------- Datenfeld zur bernahme der Messwerte --------------------*/
/*der erste Index des ARRAYs mu die Messungsnummer, der zweite die Kanalnummer*/
/*angeben,damit das PASCAL-ARRAY-Format dem Daten-Format des Echtzeit-Programmes*/
/*M8P030C.EXE entspricht.                                                      */

unsigned int data[ANZ_MESS][ANZ_KAN];	/*Array zum Ablegen der Messwerte*/

struct param {
   unsigned char status;     		/*Status des Echtzeit-Programmes*/
   unsigned int  abtastrate;  		/* 0.4 s Zeit zwischen zwei Abtastpunkten */
   unsigned char modul;       		/*Modulsteckplatz*/
   unsigned char anzkan;      		/*Anzahl zu messender Kanle*/
   unsigned int  anzmess;     		/*Anzahl von Messungen pro Kanal*/
} parameter;

unsigned int  control;                  /*Variable fr Rckgabewerte von ML8BIB*/
unsigned char b;

int    	      graphdriver;            	/*- Hilfsvariablen fr Grafikdarstellung -*/
int    	      graphmode;              	/*                  "                     */
float         fy;                       /*                  "                     */
unsigned int  x;                        /*                  "                     */
unsigned int  MaxY;                     /*                  "                     */
unsigned int  MaxX;                     /*                  "                     */
char          c;                        /*                  "                     */
unsigned char k;                        /*                  "                     */

/*---------------------- Fehlerbehandlungsroutine --------------------------*/
/*Die Fehlerbehandlungsroutine verwendet die Beispielroutine 'errorhandler '*/
/*aus dem Beispielprogramm 'DEMO1.C'.					    */
void errorhandler()
{
   struct ml8_error_info_type  error_info, *err_p=&error_info;
   int	i;
   char	s80[81];
   char	ch;
   unsigned int	w;


   ml8_get_error_info(err_p);

   if (err_p->mark==0) return;

   printf("\n\n\nBenutzer-eigene Fehlerbehandlung\n\n");

   if (err_p->ok)
   {
      printf("Die Fehlerbehandlung wurde grundlos aufgerufen\n");
      printf("(mglicherweise direkt vom C-Programm, statt von der Bibliothek)\n");
      getch();
      return;
   }
   if (err_p->errorclass==5)
   {
      w=ml8_get_message();

      printf("Die Fehlermeldung wurde durch einen Errorrequest\n");
      printf("von der MODULAR-4 Karte ausgelst\n\n");
      printf("Fehler-Request von Karte: ");
      printf("     SRQ: %X>H\n",w);
      printf("Bitte eine Taste drcken ");

      getch();
      return;
   }
   printf("\n");
   if (err_p->statusmap & 128 > 0)
   {
      printf("Kommunikationsversuch ohne Initialisierung der \n");
      printf("Bibliothek fr die angewhlte Karte\n");
      printf("Verwenden Sie ml8_start oder ml4_reset\n");
      printf("Programm angehalten\n");
      abort();
   }
   if (err_p->statusmap & 1 > 0)
   {
   printf("Kommunikationsversuch ohne ausreichende Fehlerbehandlung.\n");
   printf("Verwenden Sie ml8_clear_error in der Fehlerbehandlungsroutine\n");
   printf("Die Fehlervariablen enthalten noch den ursprnglichen Fehler.\n\n");
   }
   ml8_get_error_message(s80);
   printf("Fehlernachricht: %s\n",s80);
   printf("Fehlerklasse %d     ",err_p->errorclass);
   printf("Fehler-Code %d\n",err_p->errorcode);
   printf("Fehler aufgetreten bei Ausfhrung von Makro-Befehl %XH\n",err_p->outwords[0]);
   printf("Schnittstellenstatus (Basisadresse) %XH\n",err_p->statusport);

   printf("\nFehler aufgetreten nach Markierung %d\n",err_p->mark);
   ml8_get_mark_message(s80);
   printf("Mark-Nachricht: %s\n",s80);

   printf("\nProgramm weiterfhren <RETURN> oder Abbruch <A/a> ? ");

   ch='x';
   while (strchr("Aa'\r'", ch)==0) ch=getch();
   if (strchr("Aa", ch)>0)
   {
      printf("\nProgramm wird angehalten\n");
      ml8_exit();
      abort();
   }
   else
   {
      printf("\nProgramm wird weitergefhrt\n");
      ml8_clear_error();
      ml8_get_error_message(s80);
      if (strcmp(s80, "")!=0)
      {
	 printf("ml8_clear_error() lscht die error_message nicht\n");
      }
      w=ml8_clear();
      if (w>0)
      {
	 printf("\nml8_clear ist erfolglos. Programm wird angehalten\n");
	 ml8_exit();
	 abort();
      }
   }
}



void main()
{
   /*---------------- Karte und Bibliothek initialisieren -------------------*/
   ml8_bib_startup( 0x200 );            	/*Bibliothek fr 'C' einstellen*/
   /*Bibliothek initialisieren und Reset der MODULAR-4 */
   control = ml8_reset(0,ML8_ADR,0,0,0, (ERROR_PROC_P) errorhandler,ML8_OS,10);

   if( control != 0 ) {                   	/*Initialisierungs-Fehler?*/
      printf("Fehler beim Initialisieren der MODULAR-4 Karte!\n");
      printf("Fehlerklasse: %d ; Fehlercode: %d\n", control/128, control%128 );
      printf("Programm wird angehalten!\n");
      ml8_exit();                       	/*Verwendung von ML8BIB beenden*/
      exit( 1 );
   }

   ml8_set_mark( TEXT_MODE_ON );            	/*Merker fr Textmodus einstellen*/

   /*--------------- Echtzeitprogramm laden und installieren ----------------*/
   control = ml8_transfer_and_install(MP_NAME,0,MESS_TASK,MP_NR,MP_INT,MP_FLAG,DATA_SIZE);

   if( control != 0 ) {                   	/*Ladefehler?*/
      printf("Fehler beim Laden des Programmes %s\n", MP_NAME);
      printf("Fehlerklasse: %d ; Fehlercode: %d\n", control/128, control%128 );
      printf("Programm wird angehalten!\n");
      ml8_exit();                       	/*Verwendung von ML8BIB beenden*/
      exit( 1 );
   }

   /*---------------------- Parameter einstellen ----------------------------*/
   /*Timerwert bestimmen*/
   parameter.abtastrate = (int) (((float) TIMER_TAKT / (float) ABTAST_FREQ) + 0.5 );
   parameter.modul      = MODUL;
   parameter.anzkan     = ANZ_KAN;
   parameter.anzmess    = ANZ_MESS;

   /*Die Parameter werden ab dem Parameter 'Abtastrate' gesetzt. Der Parameter*/
   /*'Status' wird vom Echtzeitprogramm beeinflut                           */

   ml8_write_par_block(MESS_TASK,1,sizeof(parameter)-1,&parameter.abtastrate);


   /*------ Initialisierung des Graphiktreibers und der Hilfsvariablen ------*/
   graphdriver = DETECT;                        /*Graphiktreiber initialisieren*/
   initgraph(&graphdriver,&graphmode,BGI_PATH); /*             "             */
   MaxY = getmaxy();                            /*Anzahl Graphikpunkte vertikal*/
   MaxX = getmaxx();                         	/*Anzahl Graphikpunkte horizontal*/
   fy = MaxY/4096.0;                           	/*Faktor zur Darstellung in Y-Richtung*/
   if( MaxX > ANZ_MESS )
      MaxX = ANZ_MESS;                          /*Sicherstellen, da nicht mehr Werte*/
   ml8_set_mark(GRAPH_MODE_ON);                 /*Merker fr Graphikmodus einstellen */

   /*----------------------- Messungen durchfhren --------------------------*/
   do {

      ml8_call_proc(MESS_TASK,2);         	/*Messung starten*/

      /*----------------- Warten auf Ende der Messung -----------------------*/
      k = 0;
      do {                             		/*'Pollen' des Statusparameters*/
	 delay(100);                     	/*100 ms warten um MODULAR-4 nicht durch*/
						/*stndige Zugriffe zu belasten        */
	 k++;                         		/*Schleifenzhler weiterzhlen*/
	ml8_read_par_block(MESS_TASK,0,1,&parameter.status);     /*Status lesen*/
      } while( parameter.status == RUNNING && k <= 20 );

      if( k > 20 ) {                       	/*Messung innerhalb 2s beendet?*/
	closegraph();
	printf("Timeout-Fehler: Messung nicht ordnungsgem beendet!\n");
	ml8_exit();                       	/*Verwendung von ML8BIB beenden*/
	exit( 1 );
      }

      if( parameter.status != FINISHED ) {  	/*Status korrekt?*/
	closegraph();
	printf("Fehler: Messung nicht ordnungsgem beendet (Status = %d)\n",
		parameter.status );
	ml8_exit();                       	/*Verwendung von ML8BIB beenden*/
	exit( 1 );
      }

      /*------------------- gemessene Daten lesen ---------------------------*/
      ml8_reset_r_pointer(MESS_TASK);          	/*Lesepointer auf Anfang des*/
						/*Datenbereichs setzen     */
      ml8_read_data_block(MESS_TASK,sizeof(data),data); /*Messdaten bertragen*/

      /*--------------- einfache Grafikausgabe der Messwerte ----------------*/
      cleardevice();                           	/*Graphikbildschirm lschen*/
      for( k=0; k<ANZ_KAN; k++ ) {
	 setcolor(k+2);                         /*Farbe einstellen, enfllt fr monochrome und s/w Bildschirme*/
	 moveto(1,MaxY-((data[0][k] * fy)+0.5));/*Startpunkt der Linie = 1. Messwert*/
	 for( x=0; x<MaxX; x++ )
	    lineto(x,MaxY-((data[x][k] * fy))+0.5);
      }

      moveto(0,MaxY-10);
      setcolor(2);                            	/*entfllt fr monochrome und S/W-Bildschirme*/
      outtext("<Taste> : Nchste Messung,  <E> : Ende");

      c = getch();    				/*Warten auf Tastendruck*/
   } while( c != 'e' && c != 'E' );             /*Programm beenden wenn 'e' gedrckt*/

   closegraph();                                /*Graphikmodus ausschalten*/
   ml8_set_mark(TEXT_MODE_ON);                  /*Merker fr Textmodus einstellen*/
   ml8_exit();                                  /*Verwendung von ML8BIB beenden*/
}




