/****************************************************************************/
/**                       Beispielprogramm fuer                            **/
/**                Analogdatenerfassung mit M-AD16-4                       **/
/**                           Version 1.1                                  **/
/**                                                                        **/
/**			   Dateiname : MESS0322.C			   **/
/**                 Erstellungsdatum : 17.03.95                            **/
/**                  Letzte nderung : 05.12.96                            **/
/**                           Status : freigegeben                         **/
/**                            Autor : Dipl. Ing. A. Horst                 **/
/**                                                                        **/
/**                                                                        **/
/**             (c) 1995, 96 SORCUS Computer GmbH Heidelberg               **/
/**                                                                        **/
/****************************************************************************/
/** Kurzbeschreibung: Das Programm installiert das Echtzeitprogramm 322H,  **/
/** misst einen Kanal mit einer Abtastrate von x kHz und zeigt die Werte   **/
/** als Grafik an.                                                         **/
/**                                                                        **/
/** ADS7809:                                                                       **/
/** Performance (MODULAR-4/486 DX2/66) :  100.00 kHz Abtastrate (1 Kanal)  **/
/**                                       Settle Time = 0s                **/
/** Performance (MODULAR-4/486 DX4/120):  125.00 kHz Abtastrate (1 Kanal)  **/
/**                                       Settle Time = 0s                **/
/**                                                                        **/
/** AD977:                                                                       **/
/** Performance (MODULAR-4/486 DX2/66) :  150.00 kHz Abtastrate (1 Kanal)  **/
/**                                       Settle Time = 0s                **/
/** Performance (MODULAR-4/486 DX4/120):  200.00 kHz Abtastrate (1 Kanal)  **/
/**                                       Settle Time = 0s                **/
/****************************************************************************/

#include <dos.h>
#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <ctype.h>
#include <graphics.h>

#include "ml8bib.h"

 /*---------- Konstanten zur Installierung des Echtzeitprogrammes ---------*/
 /* (MP soll heissen MessProgramm)                                         */
#define MP_NAME      "M8P0322.EXE" /* Name des Echtzeitprogrammes          */
#define MP_NR        0x322         /* Nummer des Echtzeitprogrammes        */
#define MP_INT       0x91          /* Interrupt des Messprogramms (TimerA) */
#define MP_FLAG      0x1982        /* Installierungsflags des Echtz.pgms   */
#define MESS_TASK    16            /* Als Task 16 installieren             */

#define ML8_ADR      0x380         /* MODULAR-4/486 Basisadresse           */
#define MODUL        2             /* M-AD16-4 auf Steckplatz ...          */
#define KANAL        2             /* Kanal 0..7                           */

#define ANZ_MESS     1000          /* 1000 Messungen, 1 Kanal              */
#define SETTLE       0x0000        /* Settle-Time = 0s                    */
#define ABTAST_FREQ  100000L       /* Abtastrate  = ... in Hz              */
				   /* Groesse des zu reserv. Datenbereichs */
#define DATA_SIZE    (long)ANZ_MESS * 2

#define WERTEBEREICH 65536.0	   /* verkleinern des Wertebereichs = ZOOM */

 /*------------------------- Statusdefinitionen ---------------------------*/
#define RUNNING    	2
#define FINISHED   	3
#define STOPPED    	4
#define ERROR      	0x80

#define ML8_OS   	"ROM"      /* Betriebssystem der MODULAR-4         */
#define BGI_PATH  	""         /* Suchpfad zu Grafikdateien            */
#define TEXT_MODE_ON  	1
#define GRAPH_MODE_ON 	2

struct param{
 unsigned char status;     	   /* Status des Echtzeit-Programms        */
 unsigned long abtastrate;  	   /* in Hz                                */
 unsigned char modul;       	   /* Modulsteckplatz                      */
 unsigned char kanal;      	   /* zu messender Kanal                   */
 unsigned int  anzmess;     	   /* Anzahl von Messungen pro Kanal       */
 unsigned int  settle;             /* Settle-Timer Wert                    */
} parameter;

unsigned int  control;             /* Variable fuer Rueckgabe v. ML8BIB    */
unsigned char b;

int    	      graphdriver;         /* Hilfsvariablen fuer Grafikdarstell.  */
int    	      graphmode;           /*                  "                   */
float         fy;                  /*                  "                   */
unsigned int  MaxY;                /*                  "                   */
unsigned int  MaxX;                /*                  "                   */

unsigned int  x;                   /*                  "                   */
char          c;                   /*                  "                   */
unsigned char k;                   /*                  "                   */
int data[ANZ_MESS];

 /*---------------------- Fehlerbehandlungsroutine ------------------------*/
 /* Die Fehlerbehandlungsroutine verwendet die Beispielroutine 'error-     */
 /* handler ' aus dem Beispielprogramm 'DEMO1.C'.		           */
void errorhandler()
{
 struct ml8_error_info_type  error_info, *err_p = &error_info;
 char   	s80[81];
 char        	ch;
 unsigned int	w;


 ml8_get_error_info(err_p);

 if(err_p->mark == 0) return;

 printf("\n\n\nBenutzer-eigene Fehlerbehandlung\n\n");

 if(err_p->ok)
   {
    printf("Die Fehlerbehandlung wurde grundlos aufgerufen\n");
    printf("(moeglicherweise direkt vom C-Programm, statt von der Bibliothek)\n");
    getch();
    return;
   }
 if(err_p->errorclass == 5)
   {
    w = ml8_get_message();
    printf("Die Fehlermeldung wurde durch einen Errorrequest\n");
    printf("von der MODULAR-4 Karte ausgeloest\n\n");
    printf("Fehler-Request von Karte: ");
    printf("     SRQ: %X>H\n", w);
    printf("Bitte eine Taste druecken ");
    getch();
    return;
   }
 printf("\n");
 if((err_p->statusmap & 128) > 0)
   {
    printf("Kommunikationsversuch ohne Initialisierung der \n");
    printf("Bibliothek fuer die angewaehlte Karte\n");
    printf("Verwenden Sie ml8_start oder ml4_reset\n");
    printf("Programm angehalten\n");
    abort();
   }
 if((err_p->statusmap & 1) > 0)
   {
    printf("Kommunikationsversuch ohne ausreichende Fehlerbehandlung.\n");
    printf("Verwenden Sie ml8_clear_error in der Fehlerbehandlungsroutine\n");
    printf("Die Fehlervariablen enthalten noch den urspruenglichen Fehler.\n\n");
   }
 ml8_get_error_message(s80);
 printf("Fehlernachricht: %s\n", s80);
 printf("Fehlerklasse %d     ", err_p->errorclass);
 printf("Fehler-Code %d\n", err_p->errorcode);
 printf("Fehler aufgetreten bei Ausfuehrung von Makro-Befehl %XH\n", err_p->outwords[0]);
 printf("Schnittstellenstatus (Basisadresse) %XH\n", err_p->statusport);

 printf("\nFehler aufgetreten nach Markierung %d\n", err_p->mark);
 ml8_get_mark_message(s80);
 printf("Mark-Nachricht: %s\n", s80);

 printf("\nProgramm weiterfuehren <RETURN> oder Abbruch <A/a> ? ");

 ch ='x';
 while (strchr("Aa'\r'", ch) == 0) ch = getch();
 if(strchr("Aa", ch) > 0)
   {
    printf("\nProgramm wird angehalten\n");
    ml8_exit();
    abort();
   }
 else
   {
    printf("\nProgramm wird weitergefuehrt\n");
    ml8_clear_error();
    ml8_get_error_message(s80);
    if(strcmp(s80, "") != 0)
      printf("ml8_clear_error() loescht die error_message nicht\n");
    w = ml8_clear();
    if(w > 0)
      {
       printf("\nml8_clear ist erfolglos. Programm wird angehalten\n");
       ml8_exit();
       abort();
      }
   }
}


void main()
{
 ml8_bib_startup(0x200);           /* Bibliothek fuer 'C' einstellen       */
				   /* Bibliothek initialisieren und Reset  */
				   /* der MODULAR-4                        */
 control = ml8_reset(0, ML8_ADR, 0, 0, 0, (ERROR_PROC_P)errorhandler, ML8_OS, 10);

 if(control != 0)
   {                   	           /* Initialisierungs-Fehler ?            */
    printf("Fehler beim Initialisieren der MODULAR-4 Karte!\n");
    printf("Fehlerklasse: %d ; Fehlercode: %d\n", control / 128, control % 128 );
    printf("Programm wird angehalten!\n");
    ml8_exit();                    /* Verwendung von ML8BIB beenden        */
    exit(1);
   }

 /*--------------- Echtzeitprogramm laden und installieren ----------------*/
 control = ml8_transfer_and_install(MP_NAME, 0, MESS_TASK, MP_NR, MP_INT, MP_FLAG, DATA_SIZE);

 if(control != 0)
   {                   	           /* Ladefehler ?                         */
    printf("Fehler beim Laden des Programmes %s\n", MP_NAME);
    printf("Fehlerklasse: %d ; Fehlercode: %d\n", control / 128, control % 128);
    printf("Programm wird angehalten!\n");
    ml8_exit();                    /* Verwendung von ML8BIB beenden        */
    exit(1);
   }

  /*---------------------- Parameter einstellen ---------------------------*/
  /* Timerwert bestimmen                                                   */
 parameter.abtastrate = ABTAST_FREQ;
 parameter.modul      = MODUL;
 parameter.kanal      = KANAL;
 parameter.anzmess    = ANZ_MESS;
 parameter.settle     = SETTLE;
 /* Die Parameter werden ab dem Parameter 'Abtastrate' gesetzt. Der Para-  */
 /* meter 'Status' wird vom Echtzeitprogramm beeinflusst.                  */

 ml8_write_par_block(MESS_TASK, 1, sizeof(parameter) - 1, &parameter.abtastrate);

 graphdriver = DETECT;             /* Graphiktreiber initialisieren 	   */
 initgraph(&graphdriver, &graphmode, BGI_PATH);

 MaxY = getmaxy();                 /* Anzahl Graphikpunkte vertikal        */
 MaxX = getmaxx();                 /* Anzahl Graphikpunkte horizontal      */
 fy = MaxY / WERTEBEREICH;         /* Faktor zur Darstellung in Y-Richtung */
 if(MaxX > ANZ_MESS)
    MaxX = ANZ_MESS;               /* Sicherstellen, dass nicht mehr Werte */
				   /* dargestellt werden 		   */



 /*----------------------- Messungen durchfuehren -------------------------*/
 do{
    ml8_call_proc(MESS_TASK, 2);   /* Messung starten (Start-Prozedur)     */

 /*-------------------- Warten auf Ende der Messung -----------------------*/
    k = 0;
    do{                            /* 'Pollen' des Statusparameters        */
       delay(10);                  /* 100 ms warten um MODULAR-4 nicht     */
				   /* durch staendige Zugriffe zu belasten */
       k++;                        /* Schleifenzaehler weiterzaehlen       */
				   /* Status lesen                         */
       ml8_read_par_block(MESS_TASK, 0, 1, &parameter.status);
      }while(parameter.status == RUNNING && k <= 120);

    if(k > 120)
      {          	           /* Messung innerhalb 2s beendet ?       */
       closegraph();
       printf("Timeout-Fehler: Messung nicht ordnungsgemaess beendet!\n");
       ml8_exit();                 /* Verwendung von ML8BIB beenden        */
       exit(1);
      }

    if(parameter.status != FINISHED)
      {                	  	   /* Status korrekt ?                     */
       closegraph();
       printf("Fehler: Messung nicht ordnungsgemaess beendet (Status = %d)\n",
	      parameter.status);
       ml8_exit();                 /* Verwendung von ML8BIB beenden        */
       exit(1);
      }

 /*---------------------- gemessene Daten lesen ---------------------------*/

				   /* Lesepointer auf Anfang des           */
				   /* Datenbereichs setzen                 */
    ml8_reset_r_pointer(MESS_TASK);
				   /* Messdaten uebertragen                */
    ml8_read_data_block(MESS_TASK, sizeof(data), data);

    setbkcolor(BLACK);
    setcolor(WHITE);
    cleardevice();                 /* Graphikbildschirm loeschen           */
    line(0, MaxY / 2 - (data[10] * fy + 0.5), MaxX, MaxY / 2 - (data[10] * fy + 0.5));


    setcolor(LIGHTGREEN);          /* Farbe einstellen                     */
				   /* Startpunkt der Linie = 1. Messwert   */
				   /* Die ersten 10 Werte werden verworfen */
				   /* (Settle-Time !)                      */
    moveto(10, MaxY / 2 - (data[10] * fy + 0.5));
    for(x = 1; x < 100; x++)      /* Kurve ziehen ...                     */
      lineto(10 + x  * 5, MaxY / 2 - (data[10+x] * fy + 0.5));

    moveto(0, MaxY - 10);
    outtext("<Taste> : Naechste Messung,  <ESC> : Ende");

    c = tolower(getch());  	   /* Warten auf Tastendruck               */
   }while(c != 27);                /* Beenden wenn 'ESC' gedrueckt         */

 closegraph();
 ml8_exit();                       /* Verwendung von ML8BIB beenden        */
}



