/***************************************************************************
*                             MESS0327.C
*      demonstrates continuous measurement and transfer of analog data on a
*      MODULAR-4/486 board;
*
*              Version:                      1.B.
*              Creation:                     27.03.1996
*              Last change:                  19.12.1996
*              Author:                       H. Klinkenborg
*                                            A. Horst
*
*    (c) by SORCUS Computer GmbH Heidelberg 1996
***************************************************************************/

/****************************************************************************
*  Description: This program is the corresponding PC-program to the real time
*        program M8P0327.C. This program measures continuously
*        analog input from up to 16 channels of the module M-AD12-16 and
*        puts the data into a buffer of the operating system OsX of the
*        MODULAR-4/486 board. The buffer is controlled and the task sends
*        SRQs to the PC, if a defined number of data is measured. These SRQs
*        are acknowledged by this PC-program by calling a function of the
*        program M8P0327 to read the data from the system buffer of the
*        MODULAR-4/486
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board
*
*  software requirements: This program requires the library ML8MAIN.LIB
*        and the header ML8BIB.H. The operating system OSX on the
*        MODULAR-4/486 board should be ML8-3A.00x or greater.
****************************************************************************/

/****************************************************************************
*  history:
*        1.A. 27.03.1996 08:40; HK
*        - creation of the file
*        1.B. 17.12.1996 15:30; AH
*        - only one task, send buffered SRQ
****************************************************************************/

/****************************************************************************
*  Includes
****************************************************************************/

#include <time.h>                      /* include the default C libraries */
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <stdlib.h>

#include "ML8BIB.H"                    /* header of SORCUS PC-library      */

#include "M8P0327.H"                   /* header of program M8P0327        */


#define SRQ_SIZE         256           /* SRQ_SIZE                         */
#define NO_OF_CYCLES     10000         /* NO_OF_CYCLES                     */
#define TIMER_C_VALUE    100L          /* timer-tic in micro-seconds       */
#define RATE             5             /* sampling rate in multiples of    */
				       /* timer tics                       */
#define MODE             1             /* mode: 0=continously, 1=single    */

#define MICROSLOT        2             /* microslot of M-AD12-16           */
#define NO_OF_CHN        16            /* no. of channels to measure       */
#define SETTLETIME       0x25          /* settletime in multip. of 62,5ns  */

				       /* MODULAR-4 setup                  */
#define ML8_MODE         1             /* use PC interrupt                 */
#define IRQ_CHANNEL      7             /* IRQ-channel 7                    */
#define ML8_ADDRESS      0x380         /* address of ML8                   */
#define ML8_CARD         0             /* card number of ML8               */
#define TIMEOUT          20            /* timeout for ML8                  */
#define OsXNAME          "ROM"         /* operating system of ML8          */

#define PGM_M8P0327      0x327         /* no. of measurement program       */
#define TASK_M8P0327     1             /* no. of task of measurem. program */
#define PROC_READ_BUF    4             /* no. of procedure, which reads    */
				       /* the system buffer                */
#define ACTIVATE_M8P0327 2             /* no. of procedure, which          */
				       /* activates program M8P0327        */


				       /* prototypes                       */
void far pascal error_handler(void);   /* error handler                    */
				       /* interrupt service handler        */
void far pascal interrupt_service(void);

				       /* variables                        */
static ulong irq_count;                /* counter for IRQs                 */
static ulong irq_count_main;           /* counter for IRQs in main program */
ushort ov_error;

short huge data_var[100][SRQ_SIZE/2];  /* data array (ring-buffer)         */

par_type_M8P0327  parameter;           /* use parameters of prg. M8P0327   */



				       /************************************/
void far pascal interrupt_service(void)/* interrupt service routine        */
{				       /************************************/
 byte   error, dummy;                  /* error and dummy variable         */
 ushort insize;                        /* no. of bytes to receive from     */
				       /* read_buf function                */
 ushort message;                       /* message from ML8                 */

 message = ml8_get_message();          /* get the message from ML8         */

 if(message == parameter.srq_word)
   {
    ml8_call_func(TASK_M8P0327,
		  PROC_READ_BUF,
		  0,
		  &dummy,
		  SRQ_SIZE,
		  &insize,
		  &data_var[irq_count % 100],
		  &error);

    irq_count++;                       /* increment counter for IRQs       */
   }
 else ov_error++;
}


				       /************************************/
int main(void)                         /* MAIN of PC-program               */
{                                      /************************************/
 char  s[80];                          /* message string                   */
 ulong NO_OF_SRQS;                     /* expected no. of SRQs             */
 FILE  *out;                           /* file into which data is stored   */
 ulong usedsize;                       /* number of valid byte in buffer   */
 ulong freesize;                       /* number of free byte in buffer    */
 byte  error;

 if((out = fopen("DATEN.DAT","wb")) == NULL)
   {
    printf("Cannot open output file.\n");
    return(1);
   }

 clrscr();
 printf("Continuous measurement and transfer of analog data\n");
 printf("(c) 1996 SORCUS Computer GmbH\n");

 ml8_bib_startup(LANGUAGE_C);          /* used language : BORLAND C++ 3.1  */
				       /* init library & reset MODULAR-4   */
 error = ml8_reset(ML8_CARD,
		   ML8_ADDRESS,
		   ML8_MODE,
		   IRQ_CHANNEL,
		   0,
		   *error_handler,
		   OsXNAME,
		   TIMEOUT);
 if(error > 0)
   {
    printf("Error during ml8_reset:\n");
    ml8_get_error_message(s);          /* get error message                */
    printf("%s\n", s);                 /* print error message              */
    ml8_exit();                        /* quit working with MODULAR-4      */
    fclose(out);
    return(1);                         /* quit program                     */
   }

 ml8_set_macro_interruptible(TRUE);
 ml8_set_service(interrupt_service);   /* define interrupt service routine */

 parameter.timer_tic = TIMER_C_VALUE;  /* 100 us interval for TIMER-C      */

				       /* change timer tic (OsX-parameter) */
 ml8_write_par_block(0, 316, 4, &parameter.timer_tic);
				       /* TI-task parameters (timer tic,   */
				       /* used timer and interrupt) must   */
				       /* be set before the first install. */
				       /* of a TI-task !                   */

				       /* install TI-task                  */
 error = ml8_transfer_and_install("M8P0327.EXE",
				   0,
				   TASK_M8P0327,
				   PGM_M8P0327,
				   0,
				   0x0983,
				   0);
 if(error > 0)
   {
    ml8_get_error_message(s);          /* get error message                */
				       /* print error message              */
    printf("Error message from ml8_transfer_and_install:\n");
    printf("Error %04Xh: %s\n", error, s);
    ml8_exit();                        /* quit working with ML8            */
    fclose(out);
    return(1);                         /* quit program                     */
   }


 ml8_suspend_requests();               /* suspend SRQs                     */

 parameter.rate = RATE;                /* sampling rate = RATE x timer-tic */
 parameter.mode = MODE;                /* duration of sampling             */
 parameter.count = NO_OF_CYCLES;       /* no. of cycles                    */
				       /* define no. of channels           */
 parameter.no_of_channels = NO_OF_CHN;
 parameter.settletime = SETTLETIME;

 parameter.srq_limit = SRQ_SIZE;       /* no. of bytes until SRQ is send   */
 parameter.srq_word = 0x0080;          /* word to be sent as SRQ to PC     */

 parameter.micro_slot = MICROSLOT;     /* M-AD12-16 slot on MODULAR-4      */

				       /* read no. of cycles from keyboard */
 printf("\n Timer value (timer tic in us)           : %ld", parameter.timer_tic);
 printf("\n No. of cycles (multiples of timer value): %ld", parameter.count);
 printf("\n First A/D-channel:                      : %d", 0);
 printf("\n No. of A/D-channels                     : %d", parameter.no_of_channels);


 NO_OF_SRQS = (long)(parameter.count * parameter.no_of_channels * 2) / SRQ_SIZE;
				       /* print no. of SRQs to screen      */
 if(parameter.mode == 1)
   {
    printf("\n No. of SRQs (expected)                  ");
    printf(": %ld (every 50 SRQ a '*' is printed)\n\n", NO_OF_SRQS);
   }
 else
   printf("\n Hit any key to stop program !\n");

				       /* set parameters of prg. M8P0327   */
 ml8_write_par_block(TASK_M8P0327, 0, sizeof(parameter), &parameter);

 irq_count = 0;                        /* init irq counts                  */
 irq_count_main = 0;
 ov_error = 0;
				       /* activate M8P0327                 */
 ml8_call_proc(TASK_M8P0327, ACTIVATE_M8P0327);

 ml8_allow_requests();                 /* allow Requests from ML8          */

 do{                                   /* check counter for IRQs           */

				       /* every 50 SRQs ...                */
    if(irq_count >= (irq_count_main + 50))
      {  			       /* write data block to disk         */
       fwrite(data_var[irq_count_main % 100],
	      parameter.srq_limit * 50,
	      1,
	      out);
       printf("*");                    /* print '*' to screen              */
       irq_count_main += 50;           /* increment count for IRQ          */
      }
   }while((!kbhit()) && ((irq_count * MODE) < NO_OF_SRQS) && (ov_error == 0));

 ml8_suspend_requests();               /* suspend requests from MODULAR-4  */

				       /* write data block to disk         */
 fwrite(data_var[irq_count_main % 100],
	parameter.srq_limit * (irq_count - irq_count_main),
	1,
	out);

 fclose(out);                          /* close file                       */

 printf("\n");                         /* print messages to screen         */
 if(irq_count == NO_OF_SRQS)
   printf(" All %ld service requests received\n", NO_OF_SRQS);
 else
   printf(" %ld service requests received\n", irq_count);

 ml8_read_par_block(TASK_M8P0327, 0, sizeof(parameter), &parameter);

 if(ov_error == 0)
   printf(" No Buffer error occurred\n");
 else
   printf(" Buffer error occurred, Buffer-error = 0x% 02Xh\n", parameter.error);


 ml8_get_buffer_status(parameter.bufnr, &freesize, &usedsize);
 printf(" The system buffer contains %d byte.\n", usedsize);

 ml8_exit();                           /* quit working with MODULAR-4      */

 return(0);
}


void far pascal error_handler(void)
{
}
