/********************************************************************
*  MLX16_P.H
*
*  This header files declares the prototypes of the SORCUS libraries
*  for following platforms:
*
*  Operating System:    DOS
*                       Windows 3.x
*
*  SORCUS Boards:       ML8 - MODULAR-4/486
*                       ML6 - Multi-COM
*                       ML2 - Multi-LAB/2
*
*
*  Authors:             Dipl.-Ing. H.Doersam
*                       Dipl-Ing. M.Haecker
*
*  Copyrights:          SORCUS Computer GmbH 1995 .. 1998
*                       All Rights Reserved
********************************************************************/



#ifndef _INC_MLX16_P
#define _INC_MLX16_P



#if defined(__cplusplus)
extern "C" {
#endif

/**************************************************************************
*      procedure declarations for MODULAR-4/486
***************************************************************************/
ML10API void   EXPORT ml8_bib_startup(ushort language);
ML10API ushort EXPORT ml8_reset(ushort card, ushort adr, ushort mode,
                                ushort IRQ_channel, ushort DMA_channel, ERROR_PROC_P
                                error_handling_proc, char *osname, ushort timeout);
ML10API ushort EXPORT ml8_start(ushort card, ushort adr, ushort mode,
                                ushort IRQ_channel, ushort DMA_channel,
                                ERROR_PROC_P error_handling_proc, ushort timeout);
ML10API void   EXPORT ml8_select_card(ushort card);
ML10API ushort EXPORT ml8_get_selected_card(void);
ML10API void   EXPORT ml8_exit(void);
ML10API void   EXPORT ml8_exit_card(ushort card);

/************************************************************
* ml8_send_macro
* send a macro command to the card
* parameter: notation see manual chapter 12, except
*            pout which is a pointer to the ram-address where
*            all answer data will be stored (CA, s, w, v, a)
*     RETURN:      = 0: no error occured
*                  !=0: error occured
************************************************************/
ML10API ushort EXPORT ml8_send_macro(byte CB, byte format, ushort n, ushort m,
                                     byte *datain, byte *dataout);

   /* configuration/test */
ML10API ushort EXPORT ml8_reacting(void);
ML10API byte   EXPORT ml8_type_check(void);
ML10API M8BOOL EXPORT ml8_prog_in_rom(ushort pgm);
ML10API void   EXPORT ml8_set_macro_interruptible(M8BOOL on_off);
ML10API void   EXPORT ml8_cache_control(byte mode);

   /* task procedures */
ML10API void   EXPORT ml8_install_task(ushort task, ushort programnr, ushort
                                       interruptnr, ulong flag, ulong datasize, ulong adr);
ML10API void   EXPORT ml8_get_task_info(ushort task, ushort callnr, ulong * result);
ML10API byte   EXPORT ml8_get_task_status(ushort task);
ML10API void   EXPORT ml8_wakeup_task(ushort task);
ML10API void   EXPORT ml8_sleep_task(ushort task);

/* activation of a TI-Task
   priority    : number of priority (0=high, 255=low)
   count       : number of calls until the task is deactivated
   intval      : time between two calls of the TI-Task in timer tics
                 (timer tick = 1ms default)
   holdoff     : delay until the first call of the TI-Task in timer tics
*/
ML10API void   EXPORT ml8_wakeup_ti_task (ushort task, byte priority,
                                          ulong count, ulong intval, ulong holdoff);
/* deactivation of a TI-Task */
ML10API void   EXPORT ml8_sleep_ti_task (ushort task);

ML10API ushort EXPORT ml8_get_pgm_installed(ushort task);
ML10API void   EXPORT ml8_call_func(ushort task, ushort func, ushort outsize,
                                    void *outdata_var, ushort maxinsize, ushort *insize,
                                    void *indata_var, byte * error);
ML10API void   EXPORT ml8_call_function(ushort task, ushort func,
                                        ushort *outsize, void *outdata_var,
                                        ushort *insize, void *indata_var, byte * error);
ML10API void   EXPORT ml8_call_io_func(ushort handle, ushort func, ushort outsize,
                                        void *outdata_var, ushort maxinsize, ushort *insize,
                                        void *indata_var, byte *error);
ML10API void   EXPORT ml8_call_proc(ushort task, ushort procedurenr);
ML10API void   EXPORT ml8_read_par_block(ushort task, ushort start, ushort size, void * data_var);
ML10API void   EXPORT ml8_write_par_block(ushort task, ushort start, ushort size, void * data_var);
ML10API void   EXPORT ml8_reset_r_pointer(ushort task);
ML10API void   EXPORT ml8_reset_w_pointer(ushort task);
ML10API void   EXPORT ml8_move_r_pointer(ushort task, long offset);
ML10API void   EXPORT ml8_move_w_pointer(ushort task, long offset);
ML10API void   EXPORT ml8_read_data_block(ushort task, ushort size, void * data_var);
ML10API void   EXPORT ml8_write_data_block(ushort task, ushort size, void * data_var);
ML10API void   EXPORT ml8_read_data (ushort task, ulong rel_ofs, ushort size, void* data);
ML10API void   EXPORT ml8_write_data (ushort task, ulong rel_ofs, ushort size, void* data);

/**************************************************************************
   new procedures; need OsX version:
     MODULAR-4/486: ml8-3A.02r
     Multi-LAB/2:   ml2-1d.11r
***************************************************************************/
/*************************************************************************
* reads a byte/word/dword from the parameter area of a task
*     task:     tasknumber
*     start:    first parameterbyte to read
*     datavar:  pointer to read dword
*
*     return: read byte/word
**************************************************************************/
ML10API byte   EXPORT ml8_read_par_byte(ushort task, ushort start);
ML10API ushort EXPORT ml8_read_par_word(ushort task, ushort start);
ML10API void   EXPORT ml8_read_par_dword(ushort task, ushort start, ulong *datavar);

/*************************************************************************
* writes a byte/word/dword to the parameter area of a task
*     task:     tasknumber
*     start:    first parameterbyte to write to
*     data:     byte/word/dword to write
**************************************************************************/
ML10API void EXPORT ml8_write_par_byte(ushort task, ushort start, byte data);
ML10API void EXPORT ml8_write_par_word(ushort task, ushort start, ushort data);
ML10API void EXPORT ml8_write_par_dword(ushort task, ushort start, ulong data);

/*************************************************************************
* reads a byte/word/dword from the data area of a task; uses the actual
* read pointer; the read pointer is inkremented by the number of written
* bytes
*     task:     tasknumber
*     datavar:  pointer to read dword
*
*     return: read byte/word
**************************************************************************/
ML10API byte   EXPORT ml8_read_data_byte(ushort task);
ML10API ushort EXPORT ml8_read_data_word(ushort task);
ML10API void   EXPORT ml8_read_data_dword(ushort task, ulong *datavar);

/*************************************************************************
* writes a byte/word/dword to the data area of a task; uses the actual
* write pointer; the write pointer is inkremented by the number of written
* bytes
*     task:     tasknumber
*     data:     byte/word/dword to write
**************************************************************************/
ML10API void EXPORT ml8_write_data_byte(ushort task, byte data);
ML10API void EXPORT ml8_write_data_word(ushort task, ushort data);
ML10API void EXPORT ml8_write_data_dword(ushort task, ulong data);

/***************************************************************************
*  set up a new buffer
*     task:        tasknumber, the buffer belongs to
*     strategy:    strategy of RAM reservation:
*                  1: from bottom; all or nothing
*                  2: from bottom; as much as possible
*                  3: from top; all or nothing
*                  4: from top; as much as possible
*     align:       alignment of buffer (see ml8_allocate_ram in manual)
*     usage:       reserved = 0
*     size:        size of the buffer
*     handle:      handle of the buffer returned by the library
*
*     return:      0: no error occured
*
*     MODULAR-4/486: OsX newer than ml8-3A.02r
*     Multi-LAB/2:   procedure is not available
***************************************************************************/
ML10API byte EXPORT ml8_create_buffer( ushort task, byte strategy, byte align,
                                       ushort usage, ulong *size, ulong *handle);

/***************************************************************************
*  deletes a buffer
*     buffer:      handle of the buffer
*  NOT YET AVAILABLE
***************************************************************************/
ML10API void   EXPORT ml8_delete_buffer( ulong buffer);

/***************************************************************************
*  deletes the contents of a buffer
*     buffer:      handle of the buffer
***************************************************************************/
ML10API void   EXPORT ml8_clear_buffer( ulong buffer);

/***************************************************************************
*  writes a data block to a ring buffer
*     buffer:      handle of the buffer
*     strategy:    strategy:
*                  bit 0 = 0: all or nothing
*                  bit 0 = 1: as much as possible
*                  bit 1    : reserved (=0)
*                  bit 2 = 0: try once
*                  bit 2 = 1: try to write until writing was possible
*     size:        size of the data block to write
*     datavar:     pointer to the data block
*
*     return:      0 : no error
*                  2 : wrong parameter in call to procedure
*                  22: another task is just writing to buffer
*                  23: another task is just reading from buffer
*                  24: not enough free space in buffer
***************************************************************************/
ML10API byte   EXPORT ml8_set_buffer( ulong buffer, byte strategy,
                                      ushort *size, void *datavar);

/***************************************************************************
*  writes a byte/word/dword to a buffer
*     buffer:      handle of the buffer
*     data:        byte/word/dword to write to buffer
*
*     return:      see 'ml8_set_buffer'
***************************************************************************/
ML10API byte   EXPORT ml8_write_buffer_byte( ulong buffer, byte data);
ML10API byte   EXPORT ml8_write_buffer_word( ulong buffer, ushort data);
ML10API byte   EXPORT ml8_write_buffer_dword( ulong buffer, ulong data);

/***************************************************************************
*  reads a data block from a buffer
*     buffer:      handle of the buffer
*     strategy:    strategy:
*                  bit 0 = 0: all or nothing
*                  bit 0 = 1: as much as possible
*                  bit 1 = 0: read buffer
*                  bit 1 = 1: view buffer (data stay in buffer)
*                  bit 2 = 0: try once
*                  bit 2 = 1: try to write until writing was possible
*     size:        size of the data block to read
*     datavar:     pointer to the data block
*
*     return:      0 : no error
*                  2 : wrong parameter in call to procedure
*                  22: another task is just writing to buffer
*                  23: another task is just reading from buffer
*                  25: not enough data in buffer
***************************************************************************/
ML10API byte   EXPORT ml8_get_buffer( ulong buffer, byte strategy,
                                      ushort *size, void *datavar);

/***************************************************************************
*  reads a byte/word/dword from a buffer
*     buffer:      handle of the buffer
*     data:        pointer to byte/word/dword to read to buffer
*
*     return:      see 'ml8_get_buffer'
***************************************************************************/
ML10API byte   EXPORT ml8_read_buffer_byte( ulong buffer, byte *data);
ML10API byte   EXPORT ml8_read_buffer_word( ulong buffer, ushort *data);
ML10API byte   EXPORT ml8_read_buffer_dword( ulong buffer, ulong *data);

/***************************************************************************
*  gets the state of a buffer
*     buffer:      handle of the buffer
*     freesize:    pointer to the number of free bytes in buffer
*     usedsize:    pointer to the number of used bytes in buffer
****************************************************************************/
ML10API void EXPORT ml8_get_buffer_status( ulong buffer, ulong *freesize, ulong *usedsize);

/***************************************************************************
*  gets the state of the buffer that was called at last by PC; if
*  freesize or usedsize is 0xFFFF, the value is greater or equal as 0xFFFF
*  (use ml8_get_buffer which uses 'long' parameters)
*  If no buffer was called by PC, the error handler is called.
*     buffer:      handle of the buffer
*     freesize:    pointer to the number of free bytes in buffer
*     usedsize:    pointer to the number of used bytes in buffer
****************************************************************************/
ML10API void EXPORT ml8_get_prev_buffer_status( ushort *freesize, ushort *usedsize);

/***************************************************************************
*  ml8_init_io: this procedure initializes the outputs of a module as it
*        given by EEPROM or initializes the base board
*     microchannel:   slot of module to initialize (0=base board)
*     optional:       0: initialize only if Bit-0 of Word-1 in the
*                        EEPROM is =1
*                     1: initialize in any case
*
*     MODULAR-4/486: OsX newer than ml8-3A.02r
*     Multi-LAB/2:   procedure is not available
****************************************************************************/
ML10API void EXPORT ml8_init_io(byte microchannel, byte optional);

/***************************************************************************
*  ml8_get_task_number: this procedure gets the tasknumber under that a
*        program is installed
*     prognumber:      number of the program
*     *installnumber:  number of installation of the program
*                      IN=0: OUT=number of installations of the program
*                      IN>0: OUT=IN:  installnumber is available
*                      IN>0: OUT!=IN: given installnumber(IN) is not
*                              installnumber(OUT) is the maximum
*     RETURN:          tasknumber of installation with number
*                      'installnumber'
*
*     MODULAR-4/486: OsX newer than ml8-3A.02r
*     Multi-LAB/2:   procedure is not available
****************************************************************************/
ML10API ushort EXPORT ml8_get_task_number(ushort prognumber, ushort *installnumber);

/***************************************************************************
*  ml8_get_int_task: this procedure gets the tasknumber thet uses a
*        a certain interrupt
*     inter:           interruptnumber
*     *task:           tasknumber that uses the interrupt
*                      if task=0xFFFF: interrupt isn't used
*
*     MODULAR-4/486: OsX newer than ml8-3A.02r
*     Multi-LAB/2:   procedure is not available
****************************************************************************/
ML10API void EXPORT ml8_get_int_task (ushort inter, ushort *task, byte *dummy);

/*************************************************************************
*   for all OsX versions
**************************************************************************/
   /* RAM and devices */
ML10API void   EXPORT ml8_set_ram_pointer(ulong adr);
ML10API void   EXPORT ml8_read_ram(ushort size, void * data_var);
ML10API void   EXPORT ml8_write_ram(ushort size, void * data_var);
ML10API void   EXPORT ml8_read_memory (ulong start_adr, ushort size, void* data_ptr);
ML10API void   EXPORT ml8_write_memory(ulong start_adr, ushort size, void* data_ptr);
ML10API ulong  EXPORT ml8_get_physical_address (ulong ulSegOff);
ML10API void   EXPORT ml8_start_pgm(void);
ML10API byte   EXPORT ml8_in8_port(ushort port);
ML10API ushort EXPORT ml8_in16_port(ushort port);
ML10API void   EXPORT ml8_out8_port(ushort port, byte data_var);
ML10API void   EXPORT ml8_out16_port(ushort port, ushort data_var);

ML10API void   EXPORT ml8_lock_async(void);    /* not implemented */
ML10API void   EXPORT ml8_lock_sync(void);     /* not implemented */
ML10API void   EXPORT ml8_unlock(void);        /* not implemented */

ML10API ushort EXPORT ml8_transfer_pgm(ushort task, ushort usage, char *pgmname,
                                       ulong * loadadr, ulong * prepadr, ulong * flag);
/************************************************************
* ml8_enable_remote_debug
*     task         tasknumber
*     pgmname      path and name of the program file
*     loadadr      startaddress of the program in memory
*     prepadr      addres of the prepare-section of the program
*     (loadadr and prepadr are returned from ml8_transfer_pgm)
*     RETURN:      = 0: no error occured
*                  !=0: error occured
* to enable remote debugging for programs downloaded with
* ml8_transfer_pgm (ml8_transfer_and_install calls this also)
************************************************************/
ML10API ushort EXPORT ml8_enable_remote_debug (ushort task, char *pgmname,
                                               ulong loadadr, ulong prepadr);

ML10API void   EXPORT ml8_allocate_ram(ushort task, ushort usage, byte strategy,
                                       byte align, ulong *size, ulong * adr);

ML10API void   EXPORT ml8_get_free_ram_size(byte align, ulong * size);
ML10API ushort EXPORT ml8_load_op(void *osname);

ML10API ushort EXPORT ml8_read_eeprom_direct(byte modul, byte reladr);
ML10API ushort EXPORT ml8_read_eeprom_copy(byte modul, byte reladr);
ML10API void   EXPORT ml8_write_eeprom_direct(byte modul, byte reladr, ushort data_var);
ML10API void   EXPORT ml8_write_eeprom_copy(byte modul, byte reladr, ushort data_var);

ML10API byte   EXPORT ml8_get_rtc_status(void);
ML10API void   EXPORT ml8_set_rtc_mode(byte mode);
ML10API void   EXPORT ml8_get_time(byte * hour, byte * min, byte * sec);
ML10API void   EXPORT ml8_get_date_and_time(byte * year, byte * month, byte * day,
                                            byte * dow, byte * hour, byte * min, byte * sec);
ML10API void   EXPORT ml8_get_date_and_time_2000(byte * year, byte * month, byte * day,
                                                 byte * dow, byte * hour, byte * min, byte * sec);
ML10API void   EXPORT ml8_set_date_and_time(byte year, byte month, byte day, byte dow,
                                            byte hour, byte min, byte sec);
ML10API void   EXPORT ml8_set_date_and_time_2000 (byte year, byte month, byte day, byte dow,
                                                  byte hour, byte min, byte sec);
ML10API ulong  EXPORT ml8_get_rtc_date_code (void);
ML10API ulong  EXPORT ml8_get_rtc_time_code (void);
ML10API void   EXPORT ml8_local_led_on(void);
ML10API void   EXPORT ml8_local_led_off(void);
ML10API byte   EXPORT ml8_get_local_led_status(void);
ML10API void   EXPORT ml8_external_led_on(void);
ML10API void   EXPORT ml8_external_led_off(void);

   /* error handling */
ML10API void   EXPORT ml8_set_error_handling(ERROR_PROC_P error_handling_proc);
ML10API void   EXPORT ml8_get_error_info(struct ml8_error_info_type *err_info );
ML10API void   EXPORT ml8_set_error_info(struct ml8_error_info_type *err_info );
ML10API void   EXPORT ml8_set_error_message(char *errormessage);
ML10API void   EXPORT ml8_get_error_message(char *errormessage);
ML10API void   EXPORT ml8_set_mark_message(char *markmessage);
ML10API void   EXPORT ml8_get_mark_message(char *markmessage);
ML10API void   EXPORT ml8_make_err_msg( char *s, byte errclass, byte errcode);
ML10API void   EXPORT ml8_set_mark(ushort mark);
ML10API ushort EXPORT ml8_get_mark(void);
ML10API void   EXPORT ml8_set_max_error_retry(ushort max);
ML10API M8BOOL EXPORT ml8_error_repeat_test(void);
ML10API void   EXPORT ml8_set_repeat_macro(M8BOOL rmp);
ML10API ushort EXPORT ml8_clear(void);
ML10API void   EXPORT ml8_clear_error(void);
ML10API ushort EXPORT ml8_produce_error(ushort errtype);
ML10API void   EXPORT ml8_call_user_error(void);
ML10API ulong  EXPORT ml8_watch_error (ushort usInfoType);

   /* interrupt handling */
ML10API void   EXPORT ml8_set_service(SERVICE_PROC_P ml8_servProc);
ML10API void   EXPORT ml8_suspend_requests(void);
ML10API void   EXPORT ml8_allow_requests(void);
ML10API ushort EXPORT ml8_get_message(void);
ML10API M8BOOL EXPORT ml8_message_available(void);
ML10API M8BOOL EXPORT ml8_poll_request(void);
   /* diverse */
ML10API ushort EXPORT ml8_hard_reset(void);
ML10API ushort EXPORT ml8_init_test(void);
ML10API ushort EXPORT ml8_init_bib(ushort card, ushort adr, ushort mode,
                                   ushort IRQ_channel, ushort DMA_channel);
ML10API void   EXPORT ml8_auto_timeout(void);
ML10API void   EXPORT ml8_change_timeout( ushort tenthsec );
ML10API ushort EXPORT ml8_propose_timeout(ushort tenthsec);
ML10API void   EXPORT ml8_set_timeout(ushort timeout);
ML10API ushort EXPORT ml8_get_timeout(void);
ML10API ushort EXPORT ml8_transfer_and_install(char *pgmname, ushort usage, ushort task,
                                               ushort programnr, ushort interruptnr,
                                               ulong flag, ulong datasize);
   /* verions */
ML10API void   EXPORT ml8_get_bib_version(char *version);
ML10API void   EXPORT ml8_get_os_version(char *version);
ML10API void   EXPORT ml8_get_lib_version(ulong* pulVersion, ulong* pulDate);
ML10API void   EXPORT ml8_get_osx_version(ulong* pulVersion, ulong* pulDate);

/**************************************************************************
*      procedure declarations for Multi-LAB/2
*   procedure declarations that are equal to MODULAR-4/486 are only
*   renamed from ml2_xxx to ml8_xxx
***************************************************************************/
ML10API ushort EXPORT ml2_reset(ushort card, ushort adr, ushort mode,
                                ushort IRQ_channel, ushort DMA_channel, ERROR_PROC_P
                                error_handling_proc, char *osname, ushort timeout);
ML10API ushort EXPORT ml2_start(ushort card, ushort adr, ushort mode,
                                ushort IRQ_channel, ushort DMA_channel,
                                ERROR_PROC_P error_handling_proc,      ushort timeout);
ML10API void   EXPORT ml2_card_check( byte *cardtype, byte *version, byte *revision);
ML10API void   EXPORT ml2_ga_version( byte *ic14, byte *ic16, byte *ic21 );
   /* license of board */
ML10API byte   EXPORT ml2_get_license( void ); /* special library version necessary */
   /* internal procedures; don't use these */
ML10API ushort EXPORT ml2_get_reserve1( void );
ML10API void EXPORT ml2_set_reserve1(ushort wert );

/**************************************************************************
*  procedure declarations for Multi-COM
*  procedure declarations that are equal to MODULAR-4/486 are only
*  renamed from ml8_xxx to ml6_xxx
***************************************************************************/
ML10API ushort EXPORT ml6_reset (ushort card, ushort adr, ushort mode,
                                 ushort IRQ_channel, ushort DMA_channel,
                                 ERROR_PROC_P error_handling_proc,
                                 char *osname, ushort timeout);
ML10API ushort EXPORT ml6_start (ushort card, ushort adr, ushort mode,
                                 ushort IRQ_channel, ushort DMA_channel,
                                 ERROR_PROC_P error_handling_proc, ushort timeout);
/**************************************************************************
*  procedure declarations for small MODULAR-4/486
*  procedure declarations that are equal to MODULAR-4/486 are only
*  renamed from ml8_xxx to ml7_xxx
***************************************************************************/
ML10API ushort EXPORT ml7_reset (ushort card, ushort adr, ushort mode,
                                 ushort IRQ_channel, ushort DMA_channel,
                                 ERROR_PROC_P error_handling_proc,
                                 char *osname, ushort timeout);
ML10API ushort EXPORT ml7_start (ushort card, ushort adr, ushort mode,
                                 ushort IRQ_channel, ushort DMA_channel,
                                 ERROR_PROC_P error_handling_proc, ushort timeout);

/****************************************************************************
*  Functions for the MODULAR Device Drivers
****************************************************************************/
ML10API void EXPORT mdd8_get_version
   (ushort usMDD, ulong* pulVersion, ulong* pulDate);
ML10API HMDD8 EXPORT mdd8_open_channel
   (ushort usMDD, ushort usStrucSize, void *pCPS);
/***********************************************************************
*  mdd8_open_channel
*
*  PURPOSE
*  -  Opens a channel that is specified by the entried CPS
*
*  PARAMETER
*  -  usMDD          Task number of MDD
*  -  usStrucSize    Size of CPS
*  -  pCPS           Points to the CPS
*        -  ulPar                                reserved parameter = 0
*        -        pfnUserFunc                user callback procedure
*        -        mode                                = 0 for PC programs
*
*  RETURN
*  -  Handle for the opened channel
*  -  If the function fails then the returned handle is = 0
***********************************************************************/
ML10API HMDD8 EXPORT mdd8_open_channel_callback
        (USHORT usMDD, USHORT usStrucSize, void *pCPS,
        ULONG ulPar, void *pfnUserFunc, USHORT mode);
ML10API void EXPORT  mdd8_read_infotext
   (ushort usMDD, ushort usTxtNr, byte ucMaxLen, char *pacInfotext);
ML10API ushort EXPORT mdd8_get_device_status
   (ushort usMDD, ushort usDevice, ushort usIndex, ushort usExtIndex);
ML10API ushort EXPORT mdd8_scan_channel
   (ushort usMDD, ushort usInfoType, HMDD8 hChannel,
        ushort usCount, void *pInfoData);
ML10API void EXPORT mdd8_reset_driver (ushort usMDD);
ML10API void EXPORT mdd8_write_channel_byte (HMDD8 hChannel, byte ucData);
ML10API void EXPORT mdd8_write_channel_word (HMDD8 hChannel, ushort usData);
ML10API void EXPORT mdd8_write_channel_short (HMDD8 hChannel, short sData);
ML10API void EXPORT mdd8_write_channel_dword (HMDD8 hChannel, ulong ulData);
ML10API ushort EXPORT mdd8_write_channel_block
   (HMDD8 hChannel, ushort usCount, void *pData);
ML10API void EXPORT mdd8_trigger_channel (HMDD8 hChannel);
ML10API byte EXPORT mdd8_read_channel_byte (HMDD8 hChannel);
ML10API ushort EXPORT mdd8_read_channel_word (HMDD8 hChannel);
ML10API short EXPORT mdd8_read_channel_short (HMDD8 hChannel);
ML10API ulong EXPORT mdd8_read_channel_dword    (HMDD8 hChannel);
ML10API ushort EXPORT mdd8_read_channel_block
   (HMDD8 hChannel, ushort usCount, void *pData);
ML10API void EXPORT mdd8_send_channel_command
   (HMDD8 hChannel, ushort usCmd);
ML10API void EXPORT mdd8_send_channel_control
   (HMDD8 hChannel, ushort usCtrl, ulong ulCtrlData);
ML10API ushort EXPORT  mdd8_send_channel_control_block
   (HMDD8 hChannel, ushort usCtrl, ushort usSize, void* pCtrlData);
ML10API ulong EXPORT mdd8_get_channel_info (HMDD8 hChannel, ushort usInfo);
ML10API ushort EXPORT mdd8_get_channel_info_block
   (HMDD8 hChannel, ushort usInfo, ushort usSize, void* pInfoData);
ML10API void EXPORT mdd8_set_channel_caption
   (HMDD8 hChannel, ushort usLen, char* pcCaption);
ML10API void EXPORT mdd8_get_channel_caption
   (HMDD8 hChannel, ushort usLen, char* pcCaption);
ML10API void EXPORT mdd8_close_channel  (HMDD8* phChannel);
ML10API ushort EXPORT mdd8_check_handle_valid (HMDD8 hChannel);
ML10API ushort EXPORT mdd8_get_diagnosis (void);


/****************************************************************************
*  Functions for Flash Eprom
****************************************************************************/
ML10API void EXPORT ml8_set_flash_mode
   (byte ucDevNo, byte ucICNo, byte ucMode, byte ucOption);
ML10API ushort EXPORT ml8_get_flash_info
   (byte ucDevNo, byte ucICNo, byte ucMode, byte ucOption);
ML10API ushort EXPORT ml8_erase_flash_sector
   (byte ucDevNo, byte ucICNo, ulong ulFirstAdr, ulong ulLastAdr);
ML10API void EXPORT ml8_program_flash
   (byte ucDevNo, byte ucICNo, ulong ulStartAdr,
   ushort usSize, void* pData);
ML10API void EXPORT ml8_read_flash
   (byte ucDevNo, byte ucICNo, ulong ulStartAdr,
   ushort usSize, void* pData);


/***********************************************************************
*  ml8_create_version_string
*
*  PURPOSE
*  -  Converts a VERSION CODE to a version string
*  -  String returns with format "AA.B.CCC (X)"
*        AA    Version (p.e. "01")
*        B     Revision (p.e. "A")
*        CCC   Version counter (p.e. "003")
*        X     Version status
*                 F  =  Free release
*                 B  =  Beta
*                 E  =  Eprom OsX release
*                 R  =  Ram Osx release
*
*  PARAMETER
*  -  pacVerStr      Points to the string variable which receives
*                    the converted version string
*  -  ulVerCode      VERSION CODE to convert
*
*  RETURN
*  -  Nothing
***********************************************************************/
ML10API void EXPORT ml8_create_version_string
   (char* pacVerStr, ulong ulVerCode);

/***********************************************************************
*  ml8_create_date_string
*
*  PURPOSE
*  -  Converts a DATE CODE to a date string
*  -  String returns with format "AAA BB/CC/DDDD"
*        AAA   Day of week (p.e. "MON")
*        BB    Day (p.e. "01")
*        CC    Month (p.e. "07")
*        DDDD  Year (p.e. "1997")
*
*  PARAMETER
*  -  pacDateStr     Points to the string variable which receives
*                    the converted date string
*  -  ulDateCode     DATE CODE to convert
*
*  RETURN
*  -  Nothing
***********************************************************************/
ML10API void EXPORT ml8_create_date_string
   (char* pacDateStr, ulong ulDateCode);

/***********************************************************************
*  ml8_create_time_string
*
*  PURPOSE
*  -  Converts a TIME CODE to a time string
*  -  String returns with format "AA:BB:CC"
*        AA    Hours (p.e. "09")
*        BB    Minutes (p.e. "05")
*        CC    Seconds (p.e. "59")
*
*  PARAMETER
*  -  pacTimeStr     Points to the string variable which receives
*                    the converted time string
*  -  ulTimeCode     TIME CODE to convert
*
*  RETURN
*  -  Nothing
***********************************************************************/
ML10API void EXPORT ml8_create_time_string
   (char* pacTimeStr, ulong ulTimeCode);

/***********************************************************************
*  ml8_create_micro_time_string
*
*  PURPOSE
*  -  Converts a MICRO TIME CODE to a time string
*  -  String returns with format "AA:BB:CC.DDDEEE"
*        AA    Hours (p.e. "09")
*        BB    Minutes (p.e. "05")
*        CC    Seconds (p.e. "59")
*        DDD   Milliseconds (p.e. "001")
*        EEE   Microseconds (p.e. "002")
*
*  PARAMETER
*  -  pacTimeStr     Points to the string variable which receives
*                    the converted time string
*  -  prcTimeCode    Points to the MICRO TIME CODE to convert
*
*  RETURN
*  -  Nothing
***********************************************************************/
ML10API void EXPORT ml8_create_micro_time_string
   (char* pacTimeStr, MICRO_TIME_CODE* prcTimeCode);

#if defined(__cplusplus)
}  // extern C
#endif



#endif // !_INC_MLX16_P
