(************+********************************************************************************
*           Demoprogramm M8SRQTST.PAS fr TURBO PASCAL 7.0 fr die Verwendung der
*           Modular-4/486 PC-Bibliothek.
*
*           Version von M8SRQTST.PAS: 1.B.    Erstellung:      06.10.1994
*                                        letzte nderung:      06.10.1995
*                                                 Autor :      MIH
*
*           Demonstriert wird insbesondere die Auslsung von Service-Requests durch
*           die Modular-4/486 Karte zum PC. Es wird das Echtzeitprogramm
*           M8P0307.EXE bentigt.
*********************************************************************************************)

USES  ML8TYPE, ML8BIB, CRT;
{$V-}
CONST
     PC_IO_PORT = $380;                        { Basisadresse 380H                              }
     IRQ_KANAL  = 7;                           { PC-IRQ-Kanal 7 wird verwendet                  }
     INT_COUNT  = 10;                          { Anzahl der Interrupts }
     BLINKRATE  = $5000;                       { Blinkrate bzw. Interruptrate }
     MODE       = 1;                           { Betriebsart der Bibliothek: mit PC-Interrupt   }
                                               { alternativ: mode = 0; --> ohne PC-Interrupt    }
     OS: STR80  = 'ROM';                       { Betriebssystem  }
     RTPROG     = '\sorcus\ml8\rt\pascal\beisp\M8P0307.EXE';    
				               { Echtzeitprogramm, das auf die Multi-LAB/2-Karte
                                                 geladen wird }

TYPE
  Parametertyp = Record
                   Status      : BYTE;         { Status des Programms                           }
		   Blink_Rate  : Word;         { Blinkrate                                      }
		   LED_Status  : BYTE;         { Zustand der LED                                }
		   LED         : BYTE;         { LED: intern oder extern                        }
		   Countdown   : WORD;         { Anzahl On-Phasen, 0 = keine, $FFFF = unendlich }
                   With_SRQ    : BOOLEAN;      { PC-Interrupt auslsen, 1 = TRUE, 0 = FALSE     }
                   SRQ_Sign    : WORD;         { Request-Zeichen                                }
                   SRQ_Pending : BOOLEAN       { Request wartet                                 }
		 end;
VAR
   irq_count     : WORD;                        { Anzahl Service-Requests                        }
   irq_count_main: WORD;                        { bei Differenz zu irq_count wird ein neue
                                                  Interrupt angezeigt }
   message       : WORD;                        { Message bei Interrupt }
   w             : WORD;                        { Fehlervariable                                 }
   s             : STRING;
   parameter     : parametertyp;                { Parameter des Programms M8P0307                }
   sema_write    : WORD;                        { Semaphore, die der Fehlerbehandlung anzeigt, ob
                                                  ob sie aus der Interrupt-Serviceroutine auf-
                                                  gerufen wurde oder nicht }
   srqerror      : WORD;                        { Zeigt an, ob in der Interrupt-Serviceroutine
                                                  ein Fehler aufgetreten ist }

(* Benutzer-Service-Funktion,
 * wird von der Bibliothek als Teil der Hardware-Interrupt-Service-Routine
 * aufgerufen.
 * !!!! Achtung: keine DOS-Funktionen aufrufen, da nicht reentrant!!!!
 * Da in der Fehler-Behandlungsfunktion "write" benutzt wird, mu ber einen
 * Schalter ausgeschlossen werden, da bei Aufruf der Fehler-
 * Behandlungsfunktion aus der Benutzer-Service-Funktion heraus
 * "write"-Befehle ausgefhrt werden.
 *)
PROCEDURE Interrupt_Service; FAR;
var
   dummy: BYTE;
BEGIN
  sema_write:=1;                               { falls aus der Interrupt-Serviceroutime heraus
                                                 die Fehlerprozedur aufgerufen wird, so ist durch
                                                 diese Semaphore sichergestellt, da keine
                                                 Bildschirmausgabe stattfindet   }
  message:= ml8_get_message;                   { SRQ-Zeichen von Modular-4/486 Karte lesen          }
  inc(irq_count);                              { Anzahl Service-Requests inkrementieren         }
  ml8_read_par_block(16, 0, sizeof(dummy), dummy); { Dummy-Lesezugriff auf die Modular-4/486 }
  sema_write:=0;
END;


PROCEDURE error_handler;   { Die Fehlerbehandlungsprozedur mu eine FAR
                             -Prozedur sein. Die ist durch das Unit-Inter-
                             face jedoch schon gewhrleistet }
VAR
    err_rec  	: ml8_error_info_type;          { Fehlerrecord             }
    w       	: WORD;                         { Fehlervariable           }
    i       	: INTEGER;                      { Laufvariable             }
    ch		: CHAR;                         { Zeichen                  }

BEGIN
    (* falls Fehlerbehandlung aus der SRQ-Routine aufgerufen wurde, darf
       keine Bildschirmausgabe stattfinden. In diesem Fall wird das
       Programm nach Abfrage der von "srqerror" in der Hauptroutine beendet *)

   if  (sema_write<>0) then begin
      srqerror:=1;
      exit;
   end;


  ml8_get_error_info(err_rec);                  { Fehlerrecord lesen       }

  writeln('Fehlercode:   ', err_rec.errorcode);
  writeln('Fehlerklasse: ', err_rec.errorclass);
  writeln('Fehlerstring: ', err_rec.errormessage);

  ml8_exit;
  halt;
END;


BEGIN
  clrscr;
  writeln('Beispielprogramm M8SRQTST.PAS       Version 1.B. 06.10.1995');

  ml8_bib_startup( LANGUAGE_PASCAL );          { Verwendete Sprache: TURBO PASCAL               }

  w := ml8_reset(0, PC_IO_PORT, MODE, IRQ_Kanal, 1, @error_handler, OS, 10);
  IF w > 0 THEN
    BEGIN
      writeln('Fehler bei ml8_reset:');
      ml8_get_error_message(s);                { Fehlermeldung der Bibliothek lesen             }
      writeln(s);                              { und auf Bildschirm schreiben                   }
      ml8_exit;                                { Treiber fr Modular-4/486 verlassen              }
      halt;                                    { Programmabbruch                                }
    END;
  ml8_get_os_version(s);
  writeln;
  writeln('Betriebssystemversion der Karte: ',s);
  writeln;
  writeln;

  ml8_set_service(@Interrupt_service);        { Adresse der Interrupt-Service-Routine eintragen }
                                         { Programm M8P0307.EXE auf die Modular-4/486 Karte laden }

  w := ml8_transfer_and_install(RTPROG, 0, 16, $307, 0, $1980, 0);
  IF w > 0 THEN
    BEGIN
      ml8_get_error_message(s);                { Fehlermeldung der Bibliothek lesen             }
      writeln('Fehlermeldung von ml8_transfer_and_install: ');
      writeln(s);                              { und auf Bildschirm schreiben                   }
      ml8_exit;                                { Treiber fr Modular-4/486 verlassen              }
      halt;                                    { Programmabbruch                                }
    END;

  with parameter do                            { Parameter fr Programm M2P0307 setzen          }
    begin
      status := 1;                             { Starten des Programms durch Status = 1         }
      blink_rate := BLINKRATE;                 { Delay zwischen Blinkphasen                     }
      led_status := 0;                         { Led ausschalten                                }
      led := 1;
      Countdown := INT_COUNT;                  { Anzahl Blinkphasen (= Anzahl SRQ's) einstellen }
      With_SRQ := TRUE;                        { SRQ's senden                                   }
      SRQ_Sign := $0080;                       { SRQ-Zeichen                                    }
      SRQ_Pending := FALSE;                    { Kein SRQ in Warteposition                      }
    end;
                                       { Parameter setzen, Programm wird dadurch auch gestartet }
  ml8_write_par_block(16, 0, sizeof(parameter), parameter);
  srqerror:=0;
  irq_count:=0;                                { Anzahl SRQ's (fr Interrupt-Service-Routine)   }
  irq_count_main:=0;                           { Anzahl SRQ's (fr Hauptprozedur    }
  ml8_allow_requests;                          { Requests von der Modular-4/486 Karte freigeben   }

  writeln('Fr jeden Interrupt ein Sternchen auf dem Bildschirm:');
  repeat
     if ((irq_count <> irq_count_main) AND (srqerror=0)) then begin
	(* wird die Blinkrate erhht, so werden nicht mehr alle Interrupts auf
	   dem Bildschirm protokolliert, da die Benutzer-Service-Funktion
	   pro Schleifendurchlauf mehrfach aufgerufen werden kann! *)
	 irq_count_main:= irq_count;
	 writeln('* Message: ', message, '  Nummer: ', irq_count);
     end;
     if srqerror<>0 then begin   (* Fehler in Interrupt-Routine aufgetreten *)
	writeln;
        writeln('   *** Fehler in Interrupt-Service-Routine aufgetreten; Abbruch ***');
	ml8_suspend_requests;    (* weitere Interrupts unterbinden *)
	ml8_exit_card(0);
	halt;
     end;
  UNTIL keypressed or (irq_count_main=Parameter.Countdown); { In Schleife bleiben bis Tastatur-
                                               Interrupt oder Anzahl gewnschter SRQ's gesendet }
  ml8_suspend_requests;                        { Requests von der Modular-4/486 Karte unterbinden }
  writeln;
  if irq_count = Parameter.Countdown then
    writeln('Alle ',irq_count,' Service Requests empfangen')
  else
    writeln('Es sind nur ', irq_count, ' Service Requests  empfangen worden');
  ml8_exit;                                   { Am Ende des Hauptprogramm mu die ml8-Bibliothek
                                                geschlossen werden. Von besonderer Bedeutung ist
			                        dies bei Verwendung von PC-Interrupt und PC-DMA }
END.
