{****************************************************************************}
{**                       Beispielprogramm fr                             **}
{**                Analogdatenerfassung mit M-AD12-16                      **}
{**                           Version 1.A.                                 **}
{**                                                                        **}
{**                 Erstellungsdatum : 15.06.94                            **}
{**                  Letzte nderung : 24.04.96 (MIH)                      **}
{**                           Status : freigegeben                         **}
{**                            Autor : M.H.                                **}
{**                                                                        **}
{**                                                                        **}
{**             (c) 1994..1996 SORCUS Computer GmbH  Heidelberg            **}
{**                                                                        **}
{****************************************************************************}
{** Kurzbeschreibung: Das Programm installiert das Echtzeitprogramm 312H,  **}
{** mit bis zu 16 Kanle und stellt sie in einer einfachen Graphik dar    **}
{** (Beenden der Messung durch die Taste 'E')                              **}
{****************************************************************************}
{*
*  Programmart: PC-Programm
*  Funktion: Beispielprogramm zur Analogdatenerfassung mit Verwendung eines
*            Echtzeitprogrammes
*  Programmiersprache: Turbo-Pascal 7.0
*  Betriebssystemversion: ML8-2F.05  und hher
*  Verwendetes Echtzeitprogramm: M8P0312.EXE
*  Zugehrige Installationsdatei: -
*  Hardware-Voraussetzungen: Modul M-AD12-16
*  Software-Voraussetzungen: Das Programm M8P0312 mu im Verzeichnis
*                            '\sorcus\ml8\rt\bc\beisp' und der
*                            BGI-Treiber mssen im aktuellen Verzeichnis
*                            stehen.
*
*  Anmerkung: Dieses Beispiel dient dazu, zu zeigen wie mit der Pascal-Biblio-
*             thek ML8BIB ein Echtzeit-Programm auf der MODULAR-4/486 instal-
*             liert und gestartet wird. Auerdem wird gezeigt, wie Daten aus dem
*             Datenbereich einer Task zum PC transferiert werden knnen (pol-
*             ling).
*             Die Grafikausgabe ist sehr einfach gehalten. Fr sptere Programme
*             empfehlen wir die Verwendung der Grafiktoolbox GA1, mit der kom-
*             fortable Grafikausgaben in einfacher Weise realisiert werden kann.
*
*  Bedienung: Nach dem Starten wird ein Reset der MODULAR-4/486 durchgefhrt und
*             das Echtzeitprogramm installiert. Danach wird die erste Messung
*             durchgefhrt.
*             Wenn das Messergebnis auf dem Bildschirm dargestellt ist, wird
*             durch Drcken einer beliebigen Taste die nchste Messung gestar-
*             tet. Mit der Taste 'E' wird das Programm beendet.
*
*  Querverweis: MESS0312.PAS, M8P0312.C
****************************************************************************}
PROGRAM mess0312;
USES CRT,graph,ml8bib,ml8error;

CONST
      {------- Konstanten zur Installierung des Echtzeitprogrammes -------}
      {(mp soll heien MessProgramm)}
      mpName = '\SORCUS\ML8\RT\BC\BEISP\M8P0312.EXE';   {Name des Echtzeitprogrammes}
      mpNr   = $312;                   {Nummer des Echtzeitprogrammes}
      mpInt  = $91;                    {Interrupt des Messprogramms (Timer-A)}
      mpFlag = $1982;                  {Installierungsflags des Echtz.pgms}
      MessTask = 16;                   {Als Task 16 installieren }

      ml8_adr     : Word = $380;       {MODULAR-4/486 Basisadresse}
      Modul       : Byte = 2;          {M-AD12-16 auf Steckplatz ... }
      AnzKan      = 3;                 {Wieviele Kanle sollen gemessen werden}
      Settle      = $0;                {Settle-Time}

      Bereich     = 0;                 { 0=+/-10 Volt, 1=+/-5 Volt, 2=+/-2,5 Volt, 3=+/-1,25 Volt,
                                        4=+/-0,625 Volt, 5=+/-0,3125 Volt
                                        6=0-10 Volt, 7=0-5 Volt, 8=0-2,5 Volt, 9=0-1,25 Volt, 10=0-0,625 Volt
                                        11=-10-0 Volt, 12=-5-0 Volt, 13=-2,5-0 Volt, 14=-1,25-0 Volt, 15=-0,625-0 Volt}
      Gain        = 0;
      Offset      = 0;
      AnzMess     = 1000;              {1000 Messungen pro Kanal}
      AbtastFreq  = 2000;              {Abtastrate = 2 kHz}
      TimerTakt   = 2.5E6;             {Timereingangstakt der ML8 = 2,5 MHz}
      DataSize : LONGINT = AnzMess*AnzKan*2;  {Gre des zu reservierenden Datenbereichs}

      {---------------------- Statusdefinitionen ----------------------------}
      RUNNING  = 2;
      FINISHED = 3;
      STOPPED  = 4;
      ERROR    = $80;

      ml8_os : Str80 = 'ROM';         {Betriebssystem der MODULAR-4}
      BGIPath  = '';                    {Suchpfad zu Grafikdateien}
      TextModeOn  = 1;
      GraphModeOn = 2;


{------------------ Datenfeld zur bernahme der Messwerte ---------------------}
{der erste Index des ARRAYs mu die Messungsnummer, der zweite die Kanalnummer }
{angeben,damit das PASCAL-ARRAY-Format dem Daten-Format des Echtzeit-Programmes}
{M8P0304.EXE entspricht.                                                       }

VAR data : Array[1..anzmess,0..anzkan-1] OF INTEGER; {Array zum Ablegen der Messwerte}

    Parameter : RECORD
                  Status     : BYTE; {Status des Echtzeit-Programmes}
                  Abtastrate : WORD; {* 0.4 s Zeit zwischen zwei Abtastpunkten}
                  Modul      : BYTE; {Modulsteckplatz}
                  AnzKan     : BYTE; {Anzahl zu messender Kanle}
                  AnzMess    : WORD; {Anzahl von Messungen pro Kanal}
                  Settle     : Byte; {Settle-Time der Multiplexer}
                  Gain       : Byte; {Korrekturfaktor fr Gain}
                  Offs       : Byte; {Korrekturfaktor fr Offset}
                  Bereich    : Word; {Messbereich (0-15)}
                END;

    control    : WORD;               { Variable fr Rckgabewerte von ML8BIB}
    b          : BYTE;

    GraphDriver : integer;           {-- Hilfsvariablen fr Grafikdarstellung --}
    GraphMode   : integer;           {                   "                      }
    fy          : REAL;              {                   "                      }
    x           : WORD;              {                   "                      }
    MaxY        : WORD;              {                   "                      }
    MaxX        : WORD;              {                   "                      }
    c           : CHAR;              {                   "                      }
    k           : BYTE;              {                   "                      }
    ch          : Char;


{----------------------- Fehlerbehandlungsroutine ---------------------------}
{Die Fehlerbehandlungsroutine verwendet die Beispielroutine 'error_handler'  }
{aus der UNIT ml8error. Sie sorgt aber dafr, da im Fehlerfall der Graphik- }
{modus ausgeschaltet wird, damit die Fehlermeldungen richtig ausgegeben      }
{werden.                                                                     }

PROCEDURE ml8fehler; FAR;                        {unbedingt 'FAR' angeben!}
VAR err_rec : ml8_error_info_type;
BEGIN
  ml8_get_error_info(err_rec);                   {Fehler-Record lesen}

  IF err_rec.mark = GraphModeOn THEN             {Graphik eingeschaltet?}
    CloseGraph;                                  {Graphikmodus verlassen}

  error_handler;                                 {Fehlermeldung ausgeben}
END;


BEGIN
  ClrScr;
  repeat
     Writeln;
     Writeln ('Basisadresse der Modular-4/486 Karte:');
     Writeln ('  1: 380h');
     Writeln ('  2: 390h');
     Writeln ('  3: 3A0h');
     Writeln ('  4: 3B0h');
     ch := Readkey;
     case ch of
      '1': ml8_adr := $380;
      '2': ml8_adr := $390;
      '3': ml8_adr := $3A0;
      '4': ml8_adr := $3B0;
      #27: HALT;
     end;
  until (ch='1') OR (ch='2') OR (ch='3') OR (ch='4');
  repeat
     Writeln;
     Writeln ('Modulsteckplatz des M-AD12-16 Moduls:');
     Writeln ('  1: Steckplatz 1');
     Writeln ('  2: Steckplatz 2');
     Writeln ('  3: Steckplatz 3');
     Writeln ('  4: Steckplatz 4');
     ch := Readkey;
     case ch of
      '1': modul := 1;
      '2': modul := 2;
      '3': modul := 3;
      '4': modul := 4;
      #27: HALT;
     end;
  until (ch='1') OR (ch='2') OR (ch='3') OR (ch='4');

  Writeln;

  Writeln ('Einen Moment bitte, die MODULAR-4 Karte wird konfiguriert.');

  {----------------- Karte und Bibliothek initialisieren --------------------}
  ml8_bib_startup(256);                {Bibliothek fr Turbo-Pascal 6.0 einstellen}
  control := ml8_reset(0,ml8_adr,0,0,0,@ml8fehler,ml8_os,10);   {Bibliothek initialisieren und}
                                                               {Reset der MODULAR-4          }
  IF control<>0 THEN                   {Initialisierungs-Fehler?}
    BEGIN
      WRITELN('Fehler beim Initialisieren der MODULAR-4 Karte!');
      WRITELN('Fehlerklasse: ',HI(control),'; Fehlercode: ',LO(control));
      WRITELN('Programm wird angehalten!');
      ml8_exit;                        {Verwendung von ML8BIB beenden}
      HALT;
    END;

  ml8_set_mark(TextModeOn);            {Merker fr Textmodus einstellen}

  {---------------- Echtzeitprogramm laden und installieren -----------------}
  control := ml8_transfer_and_install(mpName,0,MessTask,mpNr,mpInt,mpFlag,DataSize);

  IF control<>0 THEN                   {Ladefehler?}
    BEGIN
      WRITELN('Fehler beim Laden des Programmes ',mpName);
      WRITELN('Fehlerklasse: ',HI(control),'; Fehlercode: ',LO(control));
      WRITELN('Programm wird angehalten!');
      ml8_exit;                        {Verwendung von ML8BIB beenden}
      HALT;
    END;

  {----------------------- Parameter einstellen -----------------------------}
  Parameter.Abtastrate := ROUND(Timertakt/Abtastfreq); {Timerwert bestimmen}
  Parameter.Modul      := Modul;
  Parameter.AnzKan     := AnzKan;
  Parameter.AnzMess    := AnzMess;
  Parameter.Settle     := Settle;
  Parameter.Gain       := Gain;
  Parameter.Offs       := Offset;
  Parameter.Bereich    := Bereich;


  {Die Parameter werden ab dem Parameter 'Abtastrate' gesetzt. Der Parameter}
  {'Status' wird vom Echtzeitprogramm beeinflut                            }

  ml8_write_par_block(MessTask,1,SizeOF(Parameter)-1,Parameter.Abtastrate);


  {------- Initialisierung des Graphiktreibers und der Hilfsvariablen -------}
  GraphDriver := Detect;                       {Graphiktreiber initialisieren}
  InitGraph(GraphDriver,GraphMode,BGIPath);    {              "              }
  MaxY := GetMaxY;                             {Anzahl Graphikpunkte vertikal}
  MaxX := GetMaxX;                             {Anzahl Graphikpunkte horizontal}

  fy := MaxY/(2*4096);                         {Faktor zur Darstellung in Y-Richtung}
  IF MaxX>AnzMess THEN MaxX := AnzMess;        {Sicherstellen, da nicht mehr Werte }
  ml8_set_mark(GraphModeOn);                   {Merker fr Graphikmodus einstellen  }

  {------------------------ Messungen durchfhren ---------------------------}
  REPEAT

    ml8_call_proc(MessTask,2);         {Messung starten}

    {------------------ Warten auf Ende der Messung -------------------------}
    k := 0;
    REPEAT                             {'Pollen' des Statusparameters}
       Delay(100);                     {100 ms warten um MODULAR-4 nicht durch}
                                       {stndige Zugriffe zu belasten         }
       INC(k);                         {Schleifenzhler weiterzhlen}
       ml8_read_par_block(MessTask,0,1,Parameter.status);     {Status lesen}
    UNTIL (Parameter.Status<>RUNNING) OR (k>100);

    IF k>100 THEN                      {Messung innerhalb 10s beendet?}
      BEGIN
        CloseGraph;
        WRITELN('Timeout-Fehler: Messung nicht ordnungsgem beendet!');
        ml8_exit;                        {Verwendung von ML8BIB beenden}
        HALT;
      END;

    IF Parameter.Status<>FINISHED THEN   {Status korrekt?}
      BEGIN
        CloseGraph;
        WRITELN('Fehler: Messung nicht ordnungsgem beendet (Status = ',
                Parameter.Status,')');
        ml8_exit;                        {Verwendung von ML8BIB beenden}
        HALT;
      END;

    {-------------------- gemessene Daten lesen ----------------------------}
    ml8_reset_r_pointer(MessTask);                   {Lesepointer auf Anfang des}
                                                     {Datenbereichs setzen      }
    ml8_read_data_block(MessTask,SizeOf(data),data); {Messdaten bertragen}


    {---------------- einfache Grafikausgabe der Messwerte -----------------}
    ClearDevice;                                 {Graphikbildschirm lschen}
    OutTextXY(0,Round (fy*100), '4095');
    OutTextXY(0,Round (fy*2047), '2047');
    OutTextXY(0,Round (fy*4096), '0');
    OutTextXY(0,Round (fy*6143), '-2048');
    OutTextXY(0,Round (fy*8000), '-4096');

    FOR k := 0 TO AnzKan-1 DO BEGIN
      SetColor(k+2);                             {Farbe einstellen, enfllt fr monochrome und s/w Bildschirme}
      MoveTo(1,MaxY-ROUND((data[1,k]+4096) * fy));      {Startpunkt der Linie = 1. Messwert}
      FOR x := 1 TO MaxX DO
      begin
        LineTo(x,MaxY-ROUND((data[x,k]+4096) * fy));
      end;
    END;

    MoveTo(0,MaxY-10);
    SetColor(2);                                 {entfllt fr monochrome und S/W-Bildschirme}
    OutText('        <taste> : Nchste Messung,  <E> : Ende');

    REPEAT UNTIL KeyPressed;                     {Warten auf Tastendruck}
    c := ReadKey;

  UNTIL UPCASE(c)='E';                           {Programm beenden wenn 'e' gedrckt}
  Closegraph;                                    {Graphikmodus ausschalten}
  ml8_set_mark(TextModeOn);                      {Merker fr Textmodus einstellen}
  ml8_exit;                                      {Verwendung von ML8BIB beenden}
END.




